unit Curve_Dialog;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, Math, QGrids, QClipbrd, 
  IniFiles, MathsParser, QTypes, Qt;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Math, Grids, Clipbrd, IniFiles, MathsParser;
{$endif}

type
  TCurveDialog = class(TForm)
    Panel2: TPanel;
    OK: TButton;
    CancelButton: TButton;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    cbBorderStyle: TComboBox;
    edTo: TEdit;
    Panel1: TPanel;
    Image1: TImage;
    btnColor: TButton;
    Label7: TLabel;
    cbExpression: TComboBox;
    Label1: TLabel;
    Timer1: TTimer;
    ColorDialog1: TColorDialog;
    edWidth: TEdit;
    StringGrid: TStringGrid;
    Panel3: TPanel;
    chkXb: TCheckBox;
    chkXt: TCheckBox;
    chkYl: TCheckBox;
    chkYr: TCheckBox;
    chkFixed1: TCheckBox;
    chkFixed2: TCheckBox;
    chkFixed3: TCheckBox;
    chkFixed4: TCheckBox;
    chkFixed5: TCheckBox;
    chkFixed6: TCheckBox;
    chkFixed7: TCheckBox;
    chkPaint: TCheckBox;
    Label2: TLabel;
    edDivision: TEdit;
    Label3: TLabel;
    edReplace: TEdit;
    Label5: TLabel;
    edName: TEdit;
    Label9: TLabel;
    Panel4: TPanel;
    Panel6: TPanel;
    Parser: TMathsParser;
    Panel5: TPanel;
    HelpButton: TButton;
    CopyButton: TButton;
    LimitParser: TMathsParser;
    edFrom: TEdit;
    procedure OKClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure btnColorClick(Sender: TObject);
    procedure Panel1Enter(Sender: TObject);
    procedure Panel1Exit(Sender: TObject);
    procedure Panel1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Timer1Timer(Sender: TObject);
    procedure FormShow(Sender: TObject);
   // procedure StringGridKeyDown(Sender: TObject; var Key: Word;
   //   Shift: TShiftState);
    procedure chkXbClick(Sender: TObject);
    procedure chkYlClick(Sender: TObject);
    procedure chkXtClick(Sender: TObject);
    procedure chkYrClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbExpressionExit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbExpressionKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbExpressionClick(Sender: TObject);
    procedure cbExpressionKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure chkFixed1Click(Sender: TObject);
    procedure chkFixed2Click(Sender: TObject);
    procedure chkFixed3Click(Sender: TObject);
    procedure chkFixed4Click(Sender: TObject);
    procedure chkFixed5Click(Sender: TObject);
    procedure chkFixed6Click(Sender: TObject);
    procedure chkFixed7Click(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure CopyButtonClick(Sender: TObject);
    procedure StringGridKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
   // procedure edReplaceKeyUp(Sender: TObject; var Key: Word;
   //   Shift: TShiftState);
  private
    procedure GetParameters(i: Integer);
    procedure GetDependentVariable(Expression: String; i: Integer);
    procedure SetVariables;
    procedure SetHelpContextID(i: THelpContext);
  //  function GetGridEditor(Grid: TCustomGrid): TCustomEdit;
    procedure AssignParameters(i: Integer);
    function ChangeFullToHalf(str: String): String;
  public
    C_Xmin, C_Xmax: Double;
    Msg: array[1..30] of String;
    procedure MakeDots(Equation: String; i: Integer);
    procedure LinearFitting(const i:Integer);
    procedure GetLimitValue(i: Integer);
    function NonLinearFitting(const i:Integer): Boolean;
    function ConvertToAXstyle(eq: String): String;
    function RoundToDecimalSF(Value : Extended; SigFig : Integer) : Extended;
  end;

var
  CurveDialog: TCurveDialog;
  sign, numRMSEcall, OldItemIndex: Integer;
  Settler, UsersEquation : String;
  EqModified, AutoFix : Boolean;
  {$ifdef Linux}
  // For mysterious bug under Red Hat 9
  OpenendTime: Double;
  {$endif}

implementation

uses Grapher_Main, NonLinearFitting;

{$R *.xfm}

procedure TCurveDialog.FormCreate(Sender: TObject);
var
  i: Integer;
  IniFile: TIniFile;
  S, ResourceFileName: String;
  procedure GetComboBoxItems(CB: TComboBox );
  begin
    i := CB.ItemIndex;
    CB.Items.Text := StringReplace(IniFile.ReadString(S, CB.Name +  '.Items.Text', ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
    CB.ItemIndex := i;
  end;
begin
  IniFile := TIniFile.Create(IniFileName);
  with IniFile do
    try
      cbExpression.Items.Clear;
      for i := 0 to 9 do
      begin
        dumstr := ReadString('Expressions', IntToStr(i), '');
        if dumstr <> '' then
          cbExpression.Items.Add(dumstr); //    .Strings[i+1] := dumstr;
      end;
    finally
      Free;
    end;
  if cbExpression.Items.Count = 0 then
    cbExpression.Items.Add('a * x + b');
  Msg[1] := 'Drawing a curve described as follows:';
  Msg[2] := 'Target data series is ';
  Msg[3] := '&Execute';
  Msg[4] := 'Iteration';
  Msg[5] := '&View';
  Msg[6] := '&O K';
  Msg[7] := '&Cancel';
  Msg[8] := 'Because at least one x-value of the data series is negative, this equation cannot be used. Please try another';
  Msg[9] := 'Because at least one x-value of the data series is 0, this equation cannot be used. Please try another';
  Msg[10]:= 'Because at least one y-value of the data series is negative, this equation cannot be used. Please try another';
  Msg[11]:= 'You cannot edit linear equations.';
  Msg[12]:= 'You are moving to another item of this combobox.' + #10#10 + 'Would you like to copy the editing equation to clipboard?';
  Msg[13]:= 'Are you sure you would like to register this equation?';
  Msg[14]:= 'Iterated ';
  Msg[15]:= ' times.';
  Msg[16]:= 'Would you like to copy this information to clippboard?';
  Msg[17]:= 'Do you replace the initial values with the optimized values?';
  Msg[18]:= 'You cannot use variable(s) which contain "y" when y is the independent variable.';
  Msg[19]:= '&Paste';
  Msg[20]:= '&Copy';
  Msg[21]:= 'Would you like to refer syntax of expression?';
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  Font.Assign(Main.Font);
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        Caption                  := ReadString(S, 'Caption', '');
        Label9.Caption           := ReadString(S, 'Label9.Caption', '');
        Label5.Caption           := ReadString(S, 'Label5.Caption', '');
        Label1.Caption           := ReadString(S, 'Label1.Caption', '');
        Label10.Caption          := ReadString(S, 'Label10.Caption', '');
        Label11.Caption          := ReadString(S, 'Label11.Caption', '');
        Label12.Caption          := ReadString(S, 'Label12.Caption', '');
        Label7.Caption           := ReadString(S, 'Label7.Caption', '');
        Label2.Caption           := ReadString(S, 'Label2.Caption', '');
        Label3.Caption           := ReadString(S, 'Label3.Caption', '');
        CancelButton.Caption     := ReadString(S, 'CancelButton.Caption', '');
        HelpButton.Caption       := ReadString(S, 'HelpButton.Caption', '');
        Panel3.Caption           := ReadString(S, 'Panel3.Caption', '');
        chkPaint.Caption         := ReadString(S, 'chkPaint.Caption', '');
        GetComboBoxItems(cbBorderStyle);
        for i := 1 to 21 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  if IsCurvePastingReady = True then
    CopyButton.Caption := Msg[19]
  else
    CopyButton.Caption := Msg[20];
  edReplace.Font := cbExpression.Font;
end;

procedure TCurveDialog.FormShow(Sender: TObject);
begin
  ModalResult := mrCancel;
  AssignParameters(DS);
  if NumDataSeries = NDS then
    CopyButton.Enabled := False
  else
    CopyButton.Enabled := True;
  EqModified := False;
  {$ifdef MSWINDOWS}
  keybd_event(VK_HOME, 0, 0, 0);
  Keybd_event(VK_HOME, 0,KEYEVENTF_KEYUP,0);
  {$endif}
  cbExpressionClick(Sender);
 // cbExpression.SelStart := 1;
  {$ifdef Linux}
  OpenendTime := Now;
  {$endif}
end;

procedure TCurveDialog.AssignParameters(i: Integer);
var i2: Integer;
begin
  DataSeries[i] := StringReplace(DataSeries[i], '\', '/', [rfReplaceAll]);
  GetDependentVariable(Copy(DataSeries[i], Pos('=', DataSeries[i]) + 1, 500), i) ;
  edWidth.Text := FloatToStr(Line_Width[i]);
  if LineStyle[i] > 6 then LineStyle[i] := 1;
  cbBorderStyle.ItemIndex := LineStyle[i];
  Panel1.Color := LineColor[i];
  Panel1Exit(nil);
  {$ifdef Linux}
  StringGrid.Font.Name := 'times';
  {$else}
  StringGrid.Font.Name := 'Times New Roman';
  {$endif}
  StringGrid.ColWidths[0] :=  50;
  StringGrid.ColWidths[1] := 100;
  if CurveMode[i] = 1 then
  begin
    if p[i,1] = 0 then StringGrid.Cells[1,0] := '';
    if p[i,2] = 0 then StringGrid.Cells[1,1] := '';
    StringGrid.Cells[1,2] := '';
    StringGrid.Cells[0,0] := '  a =';
    StringGrid.Cells[0,1] := '  b =';
    StringGrid.Cells[0,2] := '  R =';
  //  StringGrid.Enabled := False;
  end;
  for i2 := 1 to 7 do
    StringGrid.Cells[1,i2-1] := ' ' +  FloatToStr(RoundToDecimalSF(p[i,i2], 7));
  if Xb[i] <> 0 then
    chkXb.Checked := True
  else
    chkXt.Checked := True;
  if Yl[i] <> 0 then
    chkYl.Checked := True
  else
    chkYr.Checked := True;
  chkFixed1.Checked := Pfix[i,1];
  chkFixed2.Checked := Pfix[i,2];
  chkFixed3.Checked := Pfix[i,3];
  chkFixed4.Checked := Pfix[i,4];
  chkFixed5.Checked := Pfix[i,5];
  chkFixed6.Checked := Pfix[i,6];
  chkFixed7.Checked := Pfix[i,7];
  chkPaint.Checked := PaintBelow[i];
  case CurveMode[i] of
  0:begin
      Caption := Msg[1];
      SetHelpContextID(17);
      if Precision[i] < 5 then
        Precision[i] := 50;
      Label2.Visible := False;
   //   Label5.Visible := False;
   //   Label6.Visible := False;
   //   Label9.Visible := False;
      chkFixed1.Visible := False;
      chkFixed2.Visible := False;
      chkFixed3.Visible := False;
      chkFixed4.Visible := False;
      chkFixed5.Visible := False;
      chkFixed6.Visible := False;
      chkFixed7.Visible := False;
    //  edReplace.Visible := False;
    //  edName.Visible := False;
    //  Panel4.Top := edReplace.Top;
    //  Height := Panel4.Top + Panel4.Height + 30;
      OldItemIndex := 1;
      if Pos(':\', DataSeries[i]) = 0 then
      begin
        cbExpression.Items.Strings[0] := Trim(Copy(DataSeries[i], Pos('=', DataSeries[i]) + 1, 500));
        cbExpression.ItemIndex := 0;
      end;
      cbExpressionClick(nil);
      CancelButton.Caption := Msg[7];
      SetVariables;
    end;
  1:begin
      SetHelpContextID(11);
      Caption := Msg[2] + ' ' + IntToStr(ObjDataSeries[i]) + ' - [' + ExtractFileName(DataSeries[ObjDataSeries[i]]) + '].';
      Label3.Visible := False;
      Label5.Visible := False;
      Label9.Visible := False;
      edDivision.Visible  := False;
      edReplace.Visible := False;
      edName.Visible := False;
      StringGrid.RowCount := 3;
      Label2.Visible := False;
      chkFixed1.Visible := False;
      chkFixed2.Visible := False;
      chkFixed3.Visible := False;
      chkFixed4.Visible := False;
      chkFixed5.Visible := False;
      chkFixed6.Visible := False;
      chkFixed7.Visible := False;
      cbExpression.Items.Clear;
      cbExpression.Items.Add('a * x + b');
      cbExpression.Items.Add('exp(a * x + b)');
      cbExpression.Items.Add('a * ln(x) + b');
      cbExpression.Items.Add('a * x^b');
      cbExpression.Items.Add('a / x + b');
      cbExpression.Items.Add('a * Sqrt(x) + b');
      cbExpression.ItemIndex := ReadEnd[i];
      Panel4.Top := edReplace.Top;
      Panel6.Visible := False;
      {$ifdef Linux}
      Height := Panel4.Top + Panel4.Height +  0;
      {$else}
      Height := Panel4.Top + Panel4.Height + 30;
      {$endif}
     // cbExpression.Text := 'a1 * x + a2';
      OK.Caption := Msg[3];
      CancelButton.Caption := Msg[7];
    end;
  2:begin
      SetHelpContextID(11);
      Caption := Msg[2] + ' ' + IntToStr(ObjDataSeries[i]) + ' - [' + ExtractFileName(DataSeries[ObjDataSeries[i]]) + '].';
      Label3.Caption := Msg[4];
      Label3.Left := 465;
    //  if (Precision[i] >= 1) or (Precision[i] = 0) then Precision[i] := 0.01;
      if Precision[i] < 10 then
        Precision[i] := 1000;
      OK.Caption := Msg[3];
      CancelButton.Caption := Msg[5];
      OldItemIndex := 1;
      cbExpression.Items.Strings[0] := Trim(Copy(DataSeries[i], Pos('=', DataSeries[i]) + 1, 500));
      cbExpression.ItemIndex := 0;
     // ShowMessage(cbExpression.Items.Text);
      cbExpressionClick(nil);
      SetVariables;
    end;
  end;
  edDivision.Text := FloatToStr(Precision[i]);
  GetLimitValue(i);
  {
  if (C_Xmin = 0) and (C_Xmax = 0) then
  begin
    C_Xmin := Gmin[3];
    C_Xmax := Gmax[3];
  end;
  }
  if (C_Xmax > 0) or (C_Xmax > C_Xmin) then
  begin
    edFrom.Text := CXmin[i];
    edTo.Text := CXmax[i];
  end
  else
  begin
    edFrom.Text := CXmax[i];
    edTo.Text := CXmin[i];
  end;
end;

procedure TCurveDialog.CancelButtonClick(Sender: TObject);
var i: Integer;
begin
  i := DS;
  if StringReplace(cbExpression.Text, ' ', '', [rfReplaceAll]) = '' then
  begin
    ModalResult := mrCancel;
    Exit;
  end;
  if CancelButton.Caption = Msg[5] then
  begin
    GetParameters(DS);
    MakeDots(UsersEquation, i);
    Modified[NumFigure] := 1;
    ModalResult := mrOK;
  end;
  if CancelButton.Caption = Msg[6] then
    ModalResult := mrOK;
  if CancelButton.Caption = Msg[7] then
    ModalResult := mrCancel;
  PaintBelow[i] := chkPaint.Checked;
end;

procedure TCurveDialog.btnColorClick(Sender: TObject);
begin
  ColorDialog1.Color :=  LineColor[DS];
  if ColorDialog1.Execute then
    Panel1.Color := ColorDialog1.Color;
  Panel1Exit(Sender);
end;

procedure TCurveDialog.Panel1Enter(Sender: TObject);
begin
  With Image1.Canvas do
    begin
    Pen.Color := clYellow;
    Pen.Style := psDot;
    Brush.Color := Panel1.Color;
    Rectangle(0, 0, Image1.Width, Image1.Height);
    end;
end;

procedure TCurveDialog.Panel1Exit(Sender: TObject);
begin
  With Image1.Canvas do
    begin
    Pen.Color := Panel1.Color;
    Brush.Color := Panel1.Color;
    Rectangle(0, 0, Image1.Width, Image1.Height);
    end;
end;

procedure TCurveDialog.Panel1MouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  Panel1Enter(Sender);
  Timer1.Enabled := True;
end;

procedure TCurveDialog.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  btnColorClick(Sender)
end;

procedure TCurveDialog.Timer1Timer(Sender: TObject);
begin
  Timer1.Enabled := False;
  Panel1Exit(Sender);
end;

procedure TCurveDialog.cbExpressionExit(Sender: TObject);
var
  i: Integer;
  dumstr2 : String;
begin
  SetVariables;
  if AutoFix then
  begin
    dumstr := ConvertToAXstyle(ChangeFullToHalf(cbExpression.Text));
    dumstr2:= ConvertToAXstyle(ChangeFullToHalf(edReplace.Text));
      for i := 1 to 7 do
        if (Pos('$' + IntToStr(i), dumstr) = 0) and (Pos('$' + IntToStr(i), dumstr2) = 0) then
          Pfix[DS,i] := True;
    i := DS;
    chkFixed1.Checked := Pfix[i,1];
    chkFixed2.Checked := Pfix[i,2];
    chkFixed3.Checked := Pfix[i,3];
    chkFixed4.Checked := Pfix[i,4];
    chkFixed5.Checked := Pfix[i,5];
    chkFixed6.Checked := Pfix[i,6];
    chkFixed7.Checked := Pfix[i,7];
  end;
end;

procedure TCurveDialog.SetVariables;
var
  S: String;
  i, j: Integer;
begin
  if cbExpression.Text <> '' then
  begin
    cbExpression.Text := ChangeFullToHalf(cbExpression.Text);
    edReplace.Text := ChangeFullToHalf(edReplace.Text);
    S := StringReplace(cbExpression.Text, '#', '(' + edReplace.Text + ')', [rfReplaceAll]);
   // S := ReplaceIndependentVariables(S);
    Parser.DetectVariables(S, 'x, y');
    with Parser do
    begin
      if Panel5.Caption = ' x =' then
        if (Pos('y', V1Name) > 0) or (Pos('y', V2Name) > 0) or (Pos('y', V3Name) > 0) or (Pos('y', V4Name) > 0)
        or (Pos('y', V5Name) > 0) or (Pos('y', V6Name) > 0) or (Pos('y', V7Name) > 0)
        or (Pos('Y', V1Name) > 0) or (Pos('Y', V2Name) > 0) or (Pos('Y', V3Name) > 0) or (Pos('Y', V4Name) > 0)
        or (Pos('Y', V5Name) > 0) or (Pos('Y', V6Name) > 0) or (Pos('Y', V7Name) > 0) then
        begin
          ShowMessage(Msg[18]);
          cbExpression.SetFocus;
          Exit;
        end;
      StringGrid.Cells[0,0] := '  ' + V1Name + ' =';
      StringGrid.Cells[0,1] := '  ' + V2Name + ' =';
      StringGrid.Cells[0,2] := '  ' + V3Name + ' =';
      StringGrid.Cells[0,3] := '  ' + V4Name + ' =';
      StringGrid.Cells[0,4] := '  ' + V5Name + ' =';
      StringGrid.Cells[0,5] := '  ' + V6Name + ' =';
      StringGrid.Cells[0,6] := '  ' + V7Name + ' =';
      if CurveMode[DS] = 1 then
        StringGrid.Cells[0,2] := '  R =';
      V8Name := 'x';
      for j := 1 to 6 do
        for i := 1 to 6 do
        begin
          S := StringGrid.Cells[0,j];
          if Trim(StringReplace(S, '=', '', [])) = gvName[i] then
            StringGrid.Cells[1,j] := FloatToStr(gvValue[i]);
        end;
    end;
  end
  else
    with Parser do
    begin
      StringGrid.Cells[0,0] := '     =';
      StringGrid.Cells[0,1] := '     =';
      StringGrid.Cells[0,2] := '     =';
      StringGrid.Cells[0,3] := '     =';
      StringGrid.Cells[0,4] := '     =';
      StringGrid.Cells[0,5] := '     =';
      StringGrid.Cells[0,6] := '     =';
    end;
end;

{
function TCurveDialog.ReplaceIndependentVariables(S: String): String;
var i: Integer;
begin
  S := StringReplace(S, 'exp'  , '@1', [rfReplaceAll] + [rfIgnoreCase]);
  S := StringReplace(S, 'log10', '@2', [rfReplaceAll] + [rfIgnoreCase]);
  S := StringReplace(S, 'cos'  , '@3', [rfReplaceAll] + [rfIgnoreCase]);
  S := StringReplace(S, 'tan'  , '@4', [rfReplaceAll] + [rfIgnoreCase]);
  S := StringReplace(S, 'abs'  , '@5', [rfReplaceAll] + [rfIgnoreCase]);
  S := StringReplace(S, 'erfc' , '@6', [rfReplaceAll] + [rfIgnoreCase]);
  S := StringReplace(S, 'erf'  , '@7', [rfReplaceAll] + [rfIgnoreCase]);
  S := StringReplace(S, 'arctan','@8', [rfReplaceAll] + [rfIgnoreCase]);
  S := StringReplace(S, 'arccos','@9', [rfReplaceAll] + [rfIgnoreCase]);
  S := StringReplace(S, 'arcsin','@a', [rfReplaceAll] + [rfIgnoreCase]);
  for i := 1 to 6 do
    if gvName[i] <> '' then
      S := StringReplace(S, gvName[i], gvValue[i], [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, '@1', 'exp'  , [rfReplaceAll]);
  S := StringReplace(S, '@2', 'log10', [rfReplaceAll]);
  S := StringReplace(S, '@3', 'cos'  , [rfReplaceAll]);
  S := StringReplace(S, '@4', 'tan'  , [rfReplaceAll]);
  S := StringReplace(S, '@5', 'abs'  , [rfReplaceAll]);
  S := StringReplace(S, '@6', 'erfc' , [rfReplaceAll]);
  S := StringReplace(S, '@7', 'erf'  , [rfReplaceAll]);
  S := StringReplace(S, '@8', 'arctan',[rfReplaceAll]);
  S := StringReplace(S, '@9', 'arccos',[rfReplaceAll]);
  S := StringReplace(S, '@a', 'arcsin',[rfReplaceAll]);
  Result := S;
end;
}

procedure TCurveDialog.OKClick(Sender: TObject);
var
  i: Integer;
  selIndex: Integer;
  currLine: String;
begin
  if StringReplace(cbExpression.Text, ' ', '', [rfReplaceAll]) = '' then Exit;
  i := DS;
  GetParameters(i);
  GetLimitValue(i);
  Screen.Cursor := crHourGlass;
  Case CurveMode[i] of
  1:  LinearFitting(i);
  2:  begin
     // ShowMessage('Just a moment, Please.');
      try
        if NonLinearFitting(i) = False then
        begin
          cbExpression.SetFocus;
          Screen.Cursor := crDefault;
          Exit;
        end;
      except
        cbExpression.SetFocus;
        Screen.Cursor := crDefault;
        raise;
        Exit;
      end;
      end;
  end;
  Screen.Cursor := crDefault;
  MakeDots(UsersEquation, i);
  Modified[NumFigure] := 1;
  if CurveMode[i] = 0 then
    ModalResult := mrOK;
  //---------------------------
  if CurveMode[i] = 1 then
    Exit;
  selIndex := cbExpression.ItemIndex;
  if selIndex > 0 then
  begin
    with cbExpression do
    begin
      currLine := Items[selIndex];
      Items.Delete(selIndex);
      Items.Insert(0,currLine);
      ItemIndex := 0;
    end;
  end;
  if chkPaint.Checked then
    PaintBelow[i] := True
  else
    PaintBelow[i] := False;
end;
//----------------------------------------------------------
procedure TCurveDialog.GetParameters(i: Integer);
var
  i2: Integer;
begin
  cbExpression.Text := ChangeFullToHalf(cbExpression.Text);
  edReplace.Text := ChangeFullToHalf(edReplace.Text);
  DataSeries[i] := Panel5.Caption + ' ' + cbExpression.Text + '  <' + edReplace.Text + '>  ?' + edName.Text;
  UsersEquation := cbExpression.Text; //ConvertToAXstyle(cbExpression.Text);
  //Settler := '(' + ConvertToAXstyle(edReplace.Text) + ')';
  Settler := '(' + edReplace.Text + ')';
  CXmin[i]     := StringReplace(edFrom.Text, ' ', '', [rfReplaceAll]);
  CXmax[i]     := StringReplace(edTo.Text, ' ', '', [rfReplaceAll]);
  LineStyle[i] := cbBorderStyle.ItemIndex;
  Line_Width[i]:= StrToFloat(edWidth.Text);
  Precision[i] := StrToFloat(edDivision.Text);
  if chkXb.Checked = True then
  begin
    Xb[i] := 1;
    Xt[i] := 0;
  end
  else
  begin
    Xb[i] := 0;
    Xt[i] := 1;
  end;
  if chkYl.Checked = True then
  begin
    Yl[i] := 1;
    Yr[i] := 0;
  end
  else
  begin
    Yl[i] := 0;
    Yr[i] := 1;
  end;
  if CurveMode[i] <> 1 then
    for i2 := 1 to 7 do
      P[i,i2]  := StrToFloat(StringGrid.Cells[1,i2-1]);
  {
  if chkFixed1.Checked  = True then Pfix[i,1] := 1 else Pfix[i,1] := 0;
  if chkFixed2.Checked  = True then Pfix[i,2] := 1 else Pfix[i,2] := 0;
  if chkFixed3.Checked  = True then Pfix[i,3] := 1 else Pfix[i,3] := 0;
  if chkFixed4.Checked  = True then Pfix[i,4] := 1 else Pfix[i,4] := 0;
  if chkFixed5.Checked  = True then Pfix[i,5] := 1 else Pfix[i,5] := 0;
  if chkFixed6.Checked = True then Pfix[i,6] := 1 else Pfix[i,6] := 0;
  if chkFixed7.Checked = True then Pfix[i,7] := 1 else Pfix[i,7] := 0;
  }
  LineColor[i] := Panel1.Color;
end;

function TCurveDialog.ChangeFullToHalf(str: String): String;
{$ifdef MSWINDOWS}
var  s:array[0..256] of char;
{$endif}
begin
  if Length(str) < 256 then
  begin
    {$ifdef Linux}
    Result := str;
    {$else}
    LCMapString(GetUserDefaultLCID(), LCMAP_HALFWIDTH, PChar(str), Length(str) + 1, s, 256);
    Result := s;
    {$endif}
  end
  else
    Result := str;
end;

function TCurveDialog.ConvertToAXstyle(eq: String): String;
var i: Integer;
begin
  eq := StringReplace(eq, 'exp'  , '@1', [rfReplaceAll] + [rfIgnoreCase]);
  eq := StringReplace(eq, 'log10', '@2', [rfReplaceAll] + [rfIgnoreCase]);
  eq := StringReplace(eq, 'cos'  , '@3', [rfReplaceAll] + [rfIgnoreCase]);
  eq := StringReplace(eq, 'tan'  , '@4', [rfReplaceAll] + [rfIgnoreCase]);
  eq := StringReplace(eq, 'abs'  , '@5', [rfReplaceAll] + [rfIgnoreCase]);
  eq := StringReplace(eq, 'erfc' , '@6', [rfReplaceAll] + [rfIgnoreCase]);
  eq := StringReplace(eq, 'erf'  , '@7', [rfReplaceAll] + [rfIgnoreCase]);
  eq := StringReplace(eq, 'arctan','@8', [rfReplaceAll] + [rfIgnoreCase]);
  eq := StringReplace(eq, 'arccos','@9', [rfReplaceAll] + [rfIgnoreCase]);
  eq := StringReplace(eq, 'arcsin','@a', [rfReplaceAll] + [rfIgnoreCase]);
  with Parser do
  begin
    SortLengthOfVariable;
    for i := 1 to 7 do
    begin
      if (V1Name <> '') and (RankingOfLength[1] = i) then
        eq := StringReplace(eq, Parser.V1Name, '$1', [rfReplaceAll] + [rfIgnoreCase]);
      if (V2Name <> '') and (RankingOfLength[2] = i) then
        eq := StringReplace(eq, Parser.V2Name, '$2', [rfReplaceAll] + [rfIgnoreCase]);
      if (V3Name <> '') and (RankingOfLength[3] = i) then
        eq := StringReplace(eq, Parser.V3Name, '$3', [rfReplaceAll] + [rfIgnoreCase]);
      if (V4Name <> '') and (RankingOfLength[4] = i) then
        eq := StringReplace(eq, Parser.V4Name, '$4', [rfReplaceAll] + [rfIgnoreCase]);
      if (V5Name <> '') and (RankingOfLength[5] = i) then
        eq := StringReplace(eq, Parser.V5Name, '$5', [rfReplaceAll] + [rfIgnoreCase]);
      if (V6Name <> '') and (RankingOfLength[6] = i) then
        eq := StringReplace(eq, Parser.V6Name, '$6', [rfReplaceAll] + [rfIgnoreCase]);
      if (V7Name <> '') and (RankingOfLength[7] = i) then
        eq := StringReplace(eq, Parser.V7Name, '$7', [rfReplaceAll] + [rfIgnoreCase]);
    end;
  end;
  eq := StringReplace(eq, '@1', 'exp'  , [rfReplaceAll]);
  eq := StringReplace(eq, '@2', 'log10', [rfReplaceAll]);
  eq := StringReplace(eq, '@3', 'cos'  , [rfReplaceAll]);
  eq := StringReplace(eq, '@4', 'tan'  , [rfReplaceAll]);
  eq := StringReplace(eq, '@5', 'abs'  , [rfReplaceAll]);
  eq := StringReplace(eq, '@6', 'erfc' , [rfReplaceAll]);
  eq := StringReplace(eq, '@7', 'erf'  , [rfReplaceAll]);
  eq := StringReplace(eq, '@8', 'arctan',[rfReplaceAll]);
  eq := StringReplace(eq, '@9', 'arccos',[rfReplaceAll]);
  eq := StringReplace(eq, '@a', 'arcsin',[rfReplaceAll]);
  Result := eq;
end;
//===============================================================================================================================
function TCurveDialog.NonLinearFitting(const i:Integer): Boolean;
var
  //MinOF, ObjFunc, ObjFuncBase, aBaseNew: Double;
  a : glma; //, abase, astep: glma;
  j: Integer;
  dumstr2, dumstr3: String;
begin
  {
  MinOF := 1000000000;
  numRMSEcall := 0;
  for j := 1 to 8 do
    begin
    a[j] := P[i,j];
    abase[j] := P[i,j];
    if a[j] = 0 then astep[j] := 0.001 else astep[j] := 0.01 * a[j];
    end;
  ObjFunc := Marquadt.RMSE(i, a);
  ObjFuncBase := ObjFunc;
  while (ObjFunc >= MinOF) or (MinOF - ObjFunc > ObjFunc * 0.01) do
    begin
    if MinOF - ObjFunc > ObjFunc * 0.01 then MinOF := ObjFunc;
    for j := 8 downto 1 do
      if Pfix[i,j] <> 1 then
        begin
        a[j] := a[j] + astep[j];
        ObjFunc := Marquadt.RMSE(i, a);
        if ObjFunc < MinOF then
          astep[j] := astep[j] * 1.2
        else
          begin
          a[j] := a[j] - astep[j];
          astep[j] := - astep[j] / 2;
          end;
        end;
   //--- pattern movement ---
    if ObjFunc < ObjFuncbase then
      begin
      ObjFuncbase := ObjFunc;
      for j := 8 downto 1 do
        if Pfix[i,j] <> 1 then
          begin
          aBaseNew := a[j];
          a[j] := 2 * a[j] - abase[j];
          abase[j] := aBaseNew;
          end;
      ObjFunc := Marquadt.RMSE(i, a);
      if ObjFunc > ObjFuncbase then
        for j := 1 to 8 do
          if Pfix[i,j] <> 1 then
            a[j] := abase[j]
        else
           ObjFuncbase := ObjFunc;
      end;
    if numRMSEcall > 1000 then
      begin
      ShowMessage('Give up! Termination criteria(set by "Precision") has not been attained.' + #13#10 +
                  'Fitting is aborted since number of iteration exceeded 1,000, .');
      Break;
      end;
    end;
  }
  Result := False;
  Marquadt := TMarquadt.Create(Self);
  if Marquadt.IsValid then
  begin
    Marquadt.Optimize(a);
    Result := True;
  end
  else
  begin
    Marquadt.Free;
    Exit;
  end;
  dumstr2 := FloatToStrF(Marquadt.RMSE(i, a), ffFixed, 10, 10);
  Marquadt.Free;
  dumstr := 'y = ' + cbExpression.Text + #13#10;
  if Pos('#', AnsiLowerCase(cbExpression.Text)) > 0 then
    dumstr :=  dumstr + '( # = ' + edReplace.Text + ' )' + #13#10;
  for j := 1 to 7 do
  begin
    Case j of
    1:dumstr3 := Parser.V1Name;
    2:dumstr3 := Parser.V2Name;
    3:dumstr3 := Parser.V3Name;
    4:dumstr3 := Parser.V4Name;
    5:dumstr3 := Parser.V5Name;
    6:dumstr3 := Parser.V6Name;
    7:dumstr3 := Parser.V7Name;
    end;
    if a[j] <> 0 then
      dumstr := dumstr + dumstr3 + ' = ' + FloatToStrF(a[j], ffFixed, 10, 10) + #13#10;
  end;
  dumstr := dumstr + 'RMSE = ' + dumstr2 + #13#10;
  dumstr := dumstr + Msg[14] + IntToStr(numRMSEcall) + Msg[15];
  if MessageDlg(dumstr + #13#10 + Msg[16], mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    ClipBoard.AsText := dumstr;
  if MessageDlg(Msg[17], mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    for j := 1 to 7 do
      if a[j] <> 0 then
        begin
        StringGrid.Cells[1,j-1] := ' ' + FloatToStr(RoundToDecimalSF(a[j], 7));//     ;//FloatToStrF(a[j], ffFixed, 10, 10);
        P[i,j] := a[j];
        end;
  CancelButton.Caption := Msg[6];
end;

//===============================================================================================================================

procedure TCurveDialog.MakeDots(Equation: String; i: Integer);
var
  k, j, dumint: Integer;
begin
  Equation := StringReplace(Equation, '\', '/', [rfReplaceAll]);
  dumint := Pos('?', Equation);
  if dumint <> 0 then
    Equation := Copy(Equation, 1, dumint - 1);
  Equation := StringReplace(AnsiLowerCase(Equation), '#', Settler, [rfReplaceAll]);
 // Equation := AnsiLowerCase(Equation);
  if CurveMode[i] = 0 then
    Numdata[i] := Ceil(Precision[i]) + 1
  else
    Numdata[i] := 100;
  //for k := 1 to 7 do
  //  Equation := StringReplace(Equation, 'a' + IntToStr(k), FloatToStrF(p[i,k], ffFixed, 10, 10), [rfReplaceAll]);
  try
    if Panel5.Caption = ' x =' then
      Equation := StringReplace(Equation, 'y', 'x', [rfReplaceAll]);
    Parser.Expression := Equation;
  except
    {$ifdef Linux}
    if Application.MessageBox(PChar(Msg[21]),
                              PChar(Parser.ErrMsg), [smbYes, smbNo]) = smbYes then
    {$else}
    if Application.MessageBox(PChar(Msg[21]),
                              PChar(Parser.ErrMsg), MB_YESNO + MB_ICONWARNING) = IDYES then
    {$endif}
      Parser.ShowHelp;
    SysUtils.Abort;
  end;
  Parser.V1 := p[i,1];
  Parser.V2 := p[i,2];
  Parser.V3 := p[i,3];
  Parser.V4 := p[i,4];
  Parser.V5 := p[i,5];
  Parser.V6 := p[i,6];
  Parser.V7 := p[i,7];
  GetDependentVariable(Equation, i);
  if (Panel5.Caption <> ' x =') or (CurveMode[i] <> 0) then
    if Xb[i] = 1 then
      j := 1
    else
      j := 2
  else
    if Yl[i] = 1 then
      j := 3
    else
      j := 4;
  for k := 1 to Numdata[i] do
  begin
    if logarithm[j] = 1 then
      dum := C_Xmin * Power(Power(C_Xmax / C_Xmin, 1 / (Numdata[i] - 1)), k)
    else
      dum := C_Xmin + (C_Xmax - C_Xmin) / (Numdata[i] - 1 ) * (k - 1);
    Parser.V8 := dum;
    if (Panel5.Caption = ' y =') or (CurveMode[i] <> 0) then
    begin
      GX[i,k] := dum;
      try
        GY[i,k] := Parser.Value;
      except
        GY[i,k] := 1234567890;
      end;
    end
    else
    begin
      GY[i,k] := dum;
      try
        GX[i,k] := Parser.Value;
      except
        GX[i,k] := 1234567890;
      end;
    end;
 //   dumstr := StringReplace(Equation, 'x', FloatToStr(GX[i,k]), [rfReplaceAll]);
 //   dumstr := StringReplace(dumstr, 'e' + FloatToStr(GX[i,k]) + 'p', 'exp', [rfReplaceAll]);
    //GY[i,k] := UserDefinedFunction(dumstr);
 end;
end;

//====================================================================
procedure TCurveDialog.LinearFitting(const i:Integer);
var
  AvgX, AvgY, Covar, SSqrX, SSqrY, StdX, StdY, r: Double;
  j, i2, n: Integer;
  a : glma; //FittingParameters;
  dumstr2: String ;
  x, y: array[0..1000] of Double;
begin
  n := 0; AvgX := 0; AvgY := 0; Covar := 0; SSqrX := 0; SSqrY := 0;
  DataSeries[i] := cbExpression.Text;
  j := ObjDataSeries[i];                 //  ObjDataSeries[i] means number of target dataseries here.
  ReadEnd[i] := cbExpression.ItemIndex; //  ReadEnd[i]   means equation number for linear(lized) regression.
  for i2 := 1 to NumData[j] do
    if (GX[j,i2] > C_Xmin) and (GX[j,i2] < C_Xmax) and (GX[j,i2] <> 12345678910) then
    begin
      n := n + 1;
      Case cbExpression.ItemIndex of
      0,1   : x[n] := GX[j,i2];
      2,3   :begin
               if GX[j,i2] > 0 then
                 x[n] := ln(GX[j,i2])
               else
               begin
                 ShowMessage(Msg[8]);
                 Exit;
               end;
               if (C_Xmin <= 0) and (cbExpression.ItemIndex = 3) then
                 C_Xmin := 0.000001;
             end;
      4     :begin
               if C_Xmin <= 0 then
                 C_Xmin := 0.000001;
               if GX[j,i2] <> 0 then
                 x[n] := 1 / GX[j,i2]
               else
               begin
                 ShowMessage(Msg[9]);
                 Exit;
               end;
             end;
      5:     begin
               if GX[j,i2] > 0 then
                 x[n] := Sqrt(GX[j,i2])
               else
               begin
                 ShowMessage(Msg[8]);
                 Exit;
               end;
             end;
      end;
      Case cbExpression.ItemIndex of
      0,2,4,5: y[n] := GY[j,i2];
      1,3:  begin
              if GY[j,i2] > 0 then
                y[n] := ln(GY[j,i2])
              else
              begin
                ShowMessage(Msg[10]);
                Exit;
              end;
            end;
      end;
    end;
  for i2 := 1 to n do
  begin
    AvgX := AvgX + x[i2];
    AvgY := AvgY + y[i2];
  end;
  AvgX := AvgX / n;
  AvgY := AvgY / n;
  for i2 := 1 to n do
  begin
    Covar := Covar + (x[i2] - AvgX) * (y[i2] - AvgY);
    SSqrX := SSqrX + Sqr(x[i2] - AvgX);
    SSqrY := SSqrY + Sqr(y[i2] - AvgY);
  end;
  Case cbExpression.ItemIndex of
  0..2,4,5: begin
            a[1] := Covar / SSqrX;
            a[2] := - a[1] * AvgX + AvgY;
            end;
  3:begin
    a[2] := Covar / SSqrX;
    a[1] := exp(-a[2] * AvgX + AvgY);
    end;
  end;
  StdX := Sqrt(SSqrX / n);
  StdY := Sqrt(SSqrY / n);
  r := Covar / n / (StdX * StdY);
  dumstr := '';
  for j := 1 to 2 do
  begin
    StringGrid.Cells[1,j-1] := ' ' + FloatToStrF(a[j], ffFixed, 10, 10);
    P[i,j] := a[j];
  end;
  StringGrid.Cells[1,2] := ' ' + FloatToStrF(r, ffFixed,10, 10);
  CancelButton.Caption := Msg[6];
  dumstr := 'y = ' + cbExpression.Text + #13#10;
  for j := 1 to 2 do
  begin
    Case j of
    1:dumstr2 := 'a';
    2:dumstr2 := 'b';
    end;
      dumstr := dumstr + dumstr2 + ' = ' + FloatToStrF(a[j], ffFixed, 10, 10) + #13#10;
  end;
  dumstr := dumstr + 'r = ' + StringGrid.Cells[1,2] + #13#10;
  if MessageDlg(dumstr + #13#10 + Msg[16], mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    ClipBoard.AsText := dumstr;
end;

//====================================================================
{
procedure TCurveDialog.StringGridKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  with Sender as TStringGrid do
    begin
    if (Key =  VK_RETURN) or (Key = VK_DOWN) then
      begin
      Key := 0;
      if Row = RowCount - 1 then
        begin
        Row := 2;
        if Col < ColCount - 1 then
          Col := Col + 1
        else
          Col :=  1;
        end
      else
        Row := Row + 1;
      end;
    if Key = VK_UP then
      begin
      Key := 0;
      if Row < 1 then
        begin
        Row := 1;
        if Col > 1 then
          Col := Col - 1
        else
          Col :=  1;
        end
      else
        Row := Row - 1;
      end;
    if Key = VK_LEFT then
      GetGridEditor(StringGrid).SelStart := GetGridEditor(StringGrid).SelStart - 1;
    if Key = VK_RIGHT then
      GetGridEditor(StringGrid).SelStart := GetGridEditor(StringGrid).SelStart + 1;
    if (Key = VK_HOME)  then
      GetGridEditor(StringGrid).SelStart := 0;
    if (Key = VK_DELETE)  then
      begin
      dumint := GetGridEditor(StringGrid).SelStart;
      dumstr := StringGrid.Cells[Col,Row];
      Delete(dumstr, dumint + 1, 1);
      StringGrid.Cells[Col,Row] := dumstr;
     // ShowMessage(IntToStr(GetGridEditor(StringGrid).SelStart));
      GetGridEditor(StringGrid).SelStart := dumint;
      end;
  Key := 0;
  end;
  //StringGrid.SetFocus;
end;

function TCurveDialog.GetGridEditor(Grid: TCustomGrid): TCustomEdit;
var i: Integer;
begin
  Result:=nil;
  for i:=0 to Grid.ControlCount-1 do
    if Grid.Controls[i] is TCustomEdit then
      begin
      Result:=TCustomEdit(Grid.Controls[i]);
      Exit;
      end;

end;
}

procedure TCurveDialog.chkXbClick(Sender: TObject);
begin
  chkXt.Checked := not chkXb.Checked;
end;

procedure TCurveDialog.chkXtClick(Sender: TObject);
begin
  if chkXt.Checked = True then
  begin
    chkXb.Checked  := False;
    chkPaint.Checked := False;
    chkPaint.Enabled := False;
  end
  else
  begin
    chkXb.Checked  := True;
    chkPaint.Enabled := True;
  end;
end;

procedure TCurveDialog.chkYlClick(Sender: TObject);
begin
  chkYr.Checked := not chkYl.Checked;
end;

procedure TCurveDialog.chkYrClick(Sender: TObject);
begin
  chkYl.Checked := not chkYr.Checked;
end;

procedure TCurveDialog.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  {$ifdef Linux}
  // For mysterious bug under Red Hat 9
  if Now < OpenendTime + 3 * 0.0000115 then
    SysUtils.Abort;
  {$endif}
  if (ModalResult <> mrOK) and (Length(Copy(DataSeries[DS], Pos('=', DataSeries[DS]) + 1, 500)) = 0) then
  begin
    NumDataSeries := NumDataSeries - 1;
    DS := NumDataSeries;
    NewDS := 1;
  end;
  if DS > 0 then
    MarkStyle[DS] := 0;
  cbExpression.Items.Strings[0] := cbExpression.Text + '  <' + edReplace.Text + '>  ?' +  edName.Text;
end;

procedure TCurveDialog.cbExpressionKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var i: Integer;
begin
  {$ifdef Linux}
  if (CurveMode[DS] = 1) and (Key <> Key_Home) then
  {$else}
  if (CurveMode[DS] = 1) and (Key <> VK_HOME) then
  {$endif}
  begin
    ShowMessage(Msg[11]);
    OK.SetFocus;
    Exit;
  end;
  if EqModified = True then
  begin
    {$ifdef Linux}
    if (Key = Key_Down) or (Key = Key_Up) then
      Case Application.MessageBox(PChar(Msg[12]), 'XYGrapher', [smbYes, smbNo, smbCancel]) of
      smbYes:
        begin
          Clipboard.Astext := cbExpression.Text;
          EqModified := False;
        end;
      smbCancel: SysUtils.Abort;
      end
    {$else}
    if (Key = VK_DOWN) or (Key = VK_UP) then
      Case Application.MessageBox(PChar(Msg[12]), 'XYGrapher', MB_YESNOCANCEL) of
      IDYes:
        begin
          Clipboard.Astext := cbExpression.Text;
          EqModified := False;
        end;
      IDCANCEL: Abort;
      end
    {$endif}
    else
      {$ifdef Linux}
      if Key = Key_Return then
      {$else}
      if Key = VK_RETURN then
      {$endif}
      begin
        if MessageDlg(PChar(Msg[13]), mtConfirmation, [mbYes, mbCancel], 0) = mrYes then
        begin
          for i := cbExpression.Items.Count -1 downto 2 do
          cbExpression.Items.Strings[i] := cbExpression.Items.Strings[i-1];
          GetParameters(DS);
          cbExpression.Items.Strings[1] := Copy(DataSeries[DS], Pos('=', DataSeries[DS]) + 1, 500);
          cbExpression.ItemIndex := 0;
        end;
      end
  end
  else
    EqModified := True;
  {$ifdef Linux}
  if (Key = Key_Down) or (Key = Key_Up) then
  {$else}
  if (Key = VK_Down) or (Key = VK_Up) then
  {$endif}
    EqModified := False;
end;

procedure TCurveDialog.cbExpressionClick(Sender: TObject);
var
  i, i2, i3, j, dumint: Integer;
begin
  if cbExpression.ItemIndex = OldItemIndex then
    Exit;
  OldItemIndex := cbExpression.ItemIndex;
  j := cbExpression.ItemIndex;
  cbExpression.Text := cbExpression.items.Strings[j];
  i := Pos('<', cbExpression.Text);
  i3:= Pos('?', cbExpression.Text);
  if i = 0 then
    if i3 = 0 then
      dumint := 1000
    else
      dumint := i3
  else
    dumint := i;
 // ShowMessage(IntToStr(cbExpression.ItemIndex));
  dumstr := Copy(cbExpression.Text, 1, dumint - 1);
  i2 := Pos('>', cbExpression.items.Strings[j]);
  edReplace.Text := Copy(cbExpression.items.Strings[j], dumint + 1, i2 - dumint - 1);
  //if Pos('#', cbExpression.Text) = 0 then edReplace.Enabled := False else edReplace.Enabled := True;
  if i3 <> 0 then
    edName.Text := Copy(cbExpression.items.Strings[j], i3 + 1, 1000);
  cbExpression.Items.Strings[j] := dumstr;
  cbExpression.Text := TrimRight(cbExpression.items.Strings[j]);
  cbExpression.ItemIndex := j;
end;


procedure TCurveDialog.cbExpressionKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
//var f1, f2: Integer;
begin
  GetDependentVariable(cbExpression.Text, DS);
  {$ifdef Linux}
  if (Key <> Key_Down) and (Key <> Key_Up) and (Key <> Key_Left) and (Key <> Key_Right) then
  {$else}
  if ((Key <> VK_UP) and (Key <> VK_DOWN) and (Key <> VK_LEFT) and (Key <> VK_RIGHT)) then
  {$endif}
  begin
    AutoFix := False;
    cbExpressionExit(Sender);
    AutoFix := True;
  end;
  //f1 := Length(WideString(cbExpression.Text));
  //f2 := Length(cbExpression.Text);
  {
  with cbExpression.Font do
    if f1 = f2 then
    begin
      Name := 'Times New Roman';
     // Charset := 1;
    end
    else                                                   f
    begin
      Name := 'ͣ ī';
      Charset := 128;
    end;
  }
end;

{
procedure TCurveDialog.edReplaceKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var f1, f2: Integer;
begin
  f1 := Length(WideString(edReplace.Text));
  f2 := Length(edReplace.Text);
  with edReplace1.Font do
    if f1 = f2 then
    begin
      Name := 'Times New Roman';
      Charset := 1;
    end
    else
    begin
      Name := 'ͣ ī';
      Charset := 128;
    end;
end;
}

procedure TCurveDialog.GetDependentVariable(Expression: String; i: Integer);
begin
  if (Pos('y', AnsiLowerCase(Expression)) > 0) and (CurveMode[i] = 0) then
  begin
    if Panel5.Caption = ' y =' then
    begin
      GetLimitValue(i);
      if ((C_Xmin = Gmin[1]) and (C_Xmin <> 0)) or ((C_Xmin = 0) and (C_Xmax = 0)) then
        C_Xmin := Gmin[1];
      if (C_Xmax = Gmax[1]) and (C_Xmin <> 0) or ((C_Xmin = 0) and (C_Xmax = 0)) then
        C_Xmax := Gmax[1];
      if C_Xmax > 0 then
      begin
        edFrom.Text := FloatToStr(C_Xmin);
        edTo.Text   := FloatToStr(C_Xmax);
      end
      else
      begin
        edFrom.Text := FloatToStr(C_Xmax);
        edTo.Text   := FloatToStr(C_Xmin);
      end;
    end;
    Panel5.Caption := ' x =';
  end
  else
  begin
    if Panel5.Caption = ' x =' then
    begin
      if ((C_Xmin = Gmin[3]) and (C_Xmin <> 0)) or ((C_Xmin = 0) and (C_Xmax = 0)) then
        C_Xmin := Gmin[3];
      if (C_Xmax = Gmax[3]) and (C_Xmin <> 0) or ((C_Xmin = 0) and (C_Xmax = 0)) then
        C_Xmax := Gmax[3];
      if C_Xmax > 0 then
      begin
        edFrom.Text := FloatToStr(C_Xmin);
        edTo.Text   := FloatToStr(C_Xmax);
      end
      else
      begin
        edFrom.Text := FloatToStr(C_Xmax);
        edTo.Text   := FloatToStr(C_Xmin);
      end;
    end;
   // Panel5.Caption := ' y =';
  end;
end;

procedure TCurveDialog.chkFixed1Click(Sender: TObject);
begin
  Pfix[DS,1] := chkFixed1.Checked;
end;

procedure TCurveDialog.chkFixed2Click(Sender: TObject);
begin
  Pfix[DS,2] := chkFixed2.Checked;
end;

procedure TCurveDialog.chkFixed3Click(Sender: TObject);
begin
  Pfix[DS,3] := chkFixed3.Checked;
end;

procedure TCurveDialog.chkFixed4Click(Sender: TObject);
begin
  Pfix[DS,4] := chkFixed4.Checked;
end;

procedure TCurveDialog.chkFixed5Click(Sender: TObject);
begin
  Pfix[DS,5] := chkFixed5.Checked;
end;

procedure TCurveDialog.chkFixed6Click(Sender: TObject);
begin
  Pfix[DS,6] := chkFixed6.Checked;
end;

procedure TCurveDialog.chkFixed7Click(Sender: TObject);
begin
  Pfix[DS,7] := chkFixed7.Checked;
end;

function TCurveDialog.RoundToDecimalSF(Value : Extended; SigFig : Integer) : Extended;
var
  Places: ShortInt;
  N : Extended;
begin
  if Value <> 0 then
  begin
    Places := SigFig - Ceil(log10(Abs(Value)));
    N := Power(10.0, Places);
    RoundToDecimalSF := (Round(Value * N) )/ N;
  end
  else
    RoundToDecimalSF := Value;
end;
procedure TCurveDialog.HelpButtonClick(Sender: TObject);
begin
  {$ifdef Linux}
  InvokeHelp;
  {$else}
  Keybd_Event(VK_F1, 0, 0, 0);
  {$endif}
end;

procedure TCurveDialog.SetHelpContextID(i: THelpContext);
begin
  HelpContext := i;
  HelpButton.HelpContext := i;
end;

procedure TCurveDialog.CopyButtonClick(Sender: TObject);
begin
  if IsCurvePastingReady = False then
  begin
    GetParameters(NDS);
    IsCurvePastingReady := True;
    ShowMessage('The curve has been copyed.');
  end
  else
  begin
    ObjDataSeries[NDS] := ObjDataSeries[DS];
    CurveMode[NDS]     := CurveMode[DS];
    AssignParameters(NDS);
    AutoFix := True;
    cbExpressionExit(Sender);
    AutoFix := False; // no need ?
    IsCurvePastingReady := False;
    if LANG = 128 then
      CopyButton.Caption := '&ԡ'
    else
      CopyButton.Caption := '&Copy';
  end;
end;

procedure TCurveDialog.StringGridKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  i: Integer;
  S: String;
begin
  with Sender as TStringGrid do
    {$ifdef Linux}
    if (Key =  Key_Return) or (Key = Key_Down) or (Key = Key_Up) then
    {$else}
    if (Key =  VK_RETURN) or (Key = VK_DOWN) or (Key = VK_UP) then
    {$endif}
      for i := 1 to 6 do
      begin
        S := StringGrid.Cells[0,Row];
        if (Trim(StringReplace(S, '=', '', [])) = gvName[i]) and (StrToFloat(StringGrid.Cells[1,Row]) <> gvValue[i]) then
        begin
          ShowMessage('You cannot change value of a global variable here.');
          StringGrid.Cells[1,Row] := FloatToStr(gvValue[i]);
        end;
      end;
end;

procedure TCurveDialog.GetLimitValue(i: Integer);
  function Get_Limit_Value(S: String): Double;
    function GetValue(S: String): Double;
    begin
      Result := 0;
      if S = Parser.V1Name then
        Result := Parser.V1;
      if S = Parser.V2Name then
        Result := Parser.V2;
      if S = Parser.V3Name then
        Result := Parser.V3;
      if S = Parser.V4Name then
        Result := Parser.V4;
      if S = Parser.V5Name then
        Result := Parser.V5;
      if S = Parser.V6Name then
        Result := Parser.V6;
    end;
  begin
    with LimitParser do
    begin
      DetectVariables(S, 'x,y');
      Expression := S;
      V1 := GetValue(V1Name);
      V2 := GetValue(V2Name);
      V3 := GetValue(V3Name);
      V4 := GetValue(V4Name);
      V5 := GetValue(V5Name);
      V6 := GetValue(V6Name);
      Result := Value;
    end;
  end;
begin
  if (CXmin[i] = '0') and (CXmax[i] = '0') then
  begin
    if Panel5.Caption <> ' y =' then
    begin
      C_Xmin := Gmin[3];
      C_Xmax := Gmax[3];
      CXmin[i] := FloatToStr(Gmin[3]);
      CXmax[i] := FloatToStr(Gmax[3]);
    end
    else
    begin
      C_Xmin := Gmin[1];
      C_Xmax := Gmax[1];
      CXmin[i] := FloatToStr(Gmin[1]);
      CXmax[i] := FloatToStr(Gmax[1]);
    end;
  end
  else
  begin
    Parser.V1 := p[i,1];
    Parser.V2 := p[i,2];
    Parser.V3 := p[i,3];
    Parser.V4 := p[i,4];
    Parser.V5 := p[i,5];
    Parser.V6 := p[i,6];
    Parser.V7 := p[i,7];
    try
      C_Xmin := StrToFloat(CXmin[i]);
    except
      C_Xmin := Get_Limit_Value(CXmin[i])
    end;
    try
      C_Xmax := StrToFloat(CXmax[i]);
    except
      C_Xmax := Get_Limit_Value(CXmax[i]);
    end;
  end;
end;

end.
