unit DS_list;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, StyleLine, Math, QButtons, Qt, QTypes, IniFiles,
  ExtCtrls, Buttons, Controls, StdCtrls;
{$else}  
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, StyleLine, Math, Buttons, IniFiles;
{$endif}

type
  TDSlist = class(TForm)
    ListBox: TListBox;
    btnEdit: TButton;
    btnRemove: TButton;
    Panel1: TPanel;
    StyleLine: TStyleLine;
    Image1: TImage;
    btnReadValue: TButton;
    Panel2: TPanel;
    Image2: TImage;
    btnGoUp: TBitBtn;
    btnGoDown: TBitBtn;
    Timer1: TTimer;
    procedure FormShow(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnReadValueClick(Sender: TObject);
    procedure ListBoxKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnGoUpClick(Sender: TObject);
    procedure btnGoDownClick(Sender: TObject);
    procedure ListBoxClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Timer1Timer(Sender: TObject);
  private
    procedure DrawLegendIndicator;
    procedure SwapDataSeries(Direction: Shortint);
  public
    ObjectSelecting, ForIntegral: Boolean;
    AutoSelect: Boolean;
    SelectedDS: Integer;
  end;

var
  DSlist: TDSlist;
  Msg: array[1..10] of String;
  dumint: Integer;

implementation

uses Grapher_Main, DataSeries_Dialog, Curve_Dialog;

{$R *.xfm}

//----------------------------------------------------------
procedure TDSlist.FormCreate(Sender: TObject);
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
  i: Integer;
begin
  Msg[1] := 'Please select the target data series.';
  Msg[2] := 'Select';
  Msg[3] := 'Are you sure you want to remove this data series?';
  Msg[4] := 'List of Data Series';
  Msg[5] := '&Edit';
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  Font.Assign(Main.Font);
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        btnRemove.Caption        := ReadString(S, 'btnRemove.Caption', '');
        btnReadValue.Caption     := ReadString(S, 'btnReadValue.Caption', '');
        btnGoUp.Caption          := ReadString(S, 'btnGoUp.Caption', '');
        btnGoDown.Caption        := ReadString(S, 'btnGoDown.Caption', '');
        Font.Name                := ReadString(S, 'DSlist.Font.Name', '');
        for i := 1 to 5 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  Top := 0;
  Left := 30;
  Caption := Msg[4];
  btnEdit.Caption := Msg[5];
  ObjectSelecting := False;
  ForIntegral:= False;
  AutoSelect := False;
end;
//----------------------------------------------------------
procedure TDSlist.FormShow(Sender: TObject);
var
  i, j: Integer;
begin
  if (CurveMode[DS] <> 0) and (ObjectSelecting = True) then
    dumint := 1
  else
    dumint := 0;
  ListBox.Height := 14 * (NumDataSeries - NumSplineCurve - dumint) + 5;
  Panel1.Height  := ListBox.Height;
  btnEdit.Top := ListBox.Top + ListBox.Height + 15;
  btnRemove.Top    := btnEdit.Top;
  btnReadValue.Top := btnEdit.Top;
  btnGoUp.Top      := btnEdit.Top;
  btnGoDown.Top    := btnEdit.Top;
  {$ifdef Linux}
  Height := btnEdit.Top + 45;
  {$else}
  Height := btnEdit.Top + 70;
  {$endif}
  if ObjectSelecting then
  begin
    Caption := Msg[1];
    btnEdit.Caption := Msg[2];
    btnEdit.Left := (Width - btnEdit.Width) div 2;
    btnRemove.Visible := False;
    btnReadValue.Visible := False;
    btnGoUp.Visible   := False;
    btnGoDown.Visible := False;
    dumint := (NumDataSeries - NumSplineCurve) - 1;
    if CurveMode[DS] = 0 then
      dumint := dumint + 1;
  end
  else
  begin
    Caption := Msg[4];
    btnEdit.Caption := Msg[5];
    dumint := (NumDataSeries - NumSplineCurve);
    btnEdit.Left := 16;
    btnRemove.Visible := True;
    btnReadValue.Visible := True;
    btnGoUp.Visible   := True;
    btnGoDown.Visible := True;
  end;
  with ListBox.Items do
  begin
    Clear;
    for i := 1 to dumint do
      if (Pos(Slash, DataSeries[i]) <> 0) and (Pos('*', DataSeries[i]) = 0) then
      begin
        Add(Copy(ChangeFileExt(ExtractFileName(DataSeries[i]), ''), 1, 40))
      end
      else
      begin
        DataSeries[i] := StringReplace(DataSeries[i], '\', '/', [rfReplaceAll]);
        Add(Copy(DataSeries[i], 1, 40));
      end;
  end;
 // ListBox.ItemIndex := 0;
  with Image1.Canvas do
  begin
    Brush.Style := bsSolid;
    Pen.Color   := BGColor;
    Brush.Color := BGColor;
    Rectangle(0, 0, Width, Height);
    Brush.Style := bsClear;
  end;
  DrawLegendIndicator;
  Panel2.Height := ListBox.Height;
  with Image2.Canvas do
  begin
    Brush.Style := bsSolid;
    Brush.Color := DSlist.Color;
    Rectangle(0, 0, Panel2.Width, Panel2.Height);
    Brush.Style := bsClear;
    if CurveMode[DS] > 0 then
      dumint := NumDataSeries - 1
    else
      dumint := NumDataSeries;
    for i := 1 to NumDataSeries do
    begin
      if i < 10 then
        j := TextWidth('A')
      else
        j := TextWidth('AA');
      TextOut(15 - j, 2 + 14 * (i - 1), IntToStr(i));
    end;
  end;
  btnGoUp.Enabled := False;
  btnGoDown.Enabled := False;
  if SelectedDS > 0 then
    ListBox.ItemIndex := SelectedDS - 1;
  if AutoSelect then
    Timer1.Enabled := True;
end;
//----------------------------------------------------------
procedure TDSlist.btnEditClick(Sender: TObject);
begin
  if ObjectSelecting then
    ObjDataSeries[DS] := ListBox.ItemIndex + 1
  else
    DS := ListBox.ItemIndex + 1;
  Hide;
  if not ForIntegral then
    Main.EditDataSeries;//(DS);
  DSlist.Close;
end;

//----------------------------------------------------------
procedure TDSlist.btnRemoveClick(Sender: TObject);
var
  i, i2, i3, j, dumint: Integer;
  TempLegend: array[0..20] of String;
begin
  {$ifdef Linux}
  if Application.MessageBox(PChar(Msg[3]), PChar('XYGrapher'), [smbYes, smbNo]) = smbNo then
  {$else}
  if Application.MessageBox(PChar(Msg[3]), PChar('XYGrapher'), MB_YESNO + MB_ICONQUESTION) = IDNO then
  {$endif}
    Exit;
  NumDataSeries := NumDataSeries - 1;
  for i := ListBox.ItemIndex + 1 to (NumDataSeries - NumSplineCurve) do
  begin
    DataSeries[i] := DataSeries[i+1];
    Xb[i] := Xb[i+1]; Xb[i+1] := 0;
    Xt[i] := Xt[i+1]; Xt[i+1] := 0;
    Yl[i] := Yl[i+1]; Yl[i+1] := 0;
    Yr[i] := Yr[i+1]; Yr[i+1] := 0;
    ReadStart[i] := ReadStart[i+1];
    ReadEnd[i]   := ReadEnd[i+1];
    LineStyle[i] := LineStyle[i+1];
    LineColor[i] := LineColor[i+1];
    MarkStyle[i] := MarkStyle[i+1];
    MarkColor[i] := MarkColor[i+1];
    Line_Width[i] := Line_Width[i+1];
    MarkSize[i]  := MarkSize[i+1];
    Xmult[i]     := Xmult[i+1];
    Xoffset[i]   := Xoffset[i+1];
    Ymult[i]     := Ymult[i+1];
    Yoffset[i]   := Yoffset[i+1];
    XConverter[i]:= XConverter[i+1];
    YConverter[i]:= YConverter[i+1];
    XConvert[i]  := XConvert[i+1];
    YConvert[i]  := YConvert[i+1];
    NumData[i]   := NumData[i+1];
    CurveMode[i] := CurveMode[i+1];
    ObjDataSeries[i] := ObjDataSeries[i+1];
    Precision[i] := Precision[i+1];
    CXmax[i] :=  CXmax[i+1];
    CXmin[i] :=  CXmin[i+1];
    for i2 := 1 to 8 do
    begin
      P[i,i2] := P[i+1,i2];
      Pfix[i,i2] := Pfix[i+1,i2];
    end;
    for i2 := 1 to NumData[i] do
    begin
      GX[i,i2] := GX[i+1,i2];
      GY[i,i2] := GY[i+1,i2];
    end;
  end;
  for i := (NumDataSeries - NumSplineCurve) + 1 to 10 do
  begin
    Xmult[i]   := 1;
    Xoffset[i] := 0;
    Ymult[i]   := 1;
    Yoffset[i] := 0;
    Xb[i] := 1;
    Yl[i] := 2;
    LegendLspacing[i] := 5;
    MarkSize[i] := 1.5;
    Line_Width[i] := 0.2;
    CurveMode[i] := 0;
    ObjDataSeries[i] := 0;
    Precision[i] := 0.001;
    for i2 := 1 to 8 do
    begin
      P[i,i2] := P[i+1,i2];
      Pfix[i,i2] := Pfix[i+1,i2];
    end;
  end;
  //--------------------------
  i2 := 0;
  for i := 6 to NumLegend do
  begin
    for i3 := 1 to 20 do
      TempLegend[i3] := '';
    dumstr := Legend[i];
    while dumstr <> '' do
    begin
      i2 := i2 + 1;
      TempLegend[i2] := Copy(dumstr, 1, Pos('\n', dumstr) - 1);
      dumstr := Copy(dumstr, Pos('\n', dumstr) + 2, 500);
    end;
    dumint := i2;
    Legend[i] := '';
    for j := 1 to dumint do
    begin
      if Pos('%' + IntToStr(ListBox.ItemIndex + 1), TempLegend[j]) <> 0 then
        TempLegend[j] := ''
      else
        for i3 := ListBox.ItemIndex + 1 to (NumDataSeries - NumSplineCurve) do
          TempLegend[j] := StringReplace(TempLegend[j], '%' + IntToStr(i3 + 1), '%' + IntToStr(i3), [rfReplaceAll]);
      if TempLegend[j] <> '' then
        Legend[i] := Legend[i] + TempLegend[j] + '\n';
    end;
  end;
  Main.ActivateAxes;
  Modified[NumFigure] := 1;
  FormShow(Sender);
end;
//----------------------------------------------------------
procedure TDSlist.btnGoUpClick(Sender: TObject);
begin
  SwapDataSeries(1);
end;

procedure TDSlist.btnGoDownClick(Sender: TObject);
begin
  SwapDataSeries(-1);
end;

procedure TDSlist.SwapDataSeries(Direction: Shortint);
var
  i, i2, j, n: Integer;
  x: Double;
  tempx, tempy: array[0..2000] of Double;
  B: Boolean;
  S: String;
begin
  i := ListBox.ItemIndex + 1;
  if Direction = -1 then
    n := 1
  else
    n := -1;
  S := DataSeries[i];  DataSeries[i] := DataSeries[i+n]; DataSeries[i+n] := S;
  j := Xb[i]; Xb[i] := Xb[i+n]; Xb[i+n] := j;
  j := Xt[i]; Xt[i] := Xt[i+n]; Xt[i+n] := j;
  j := Yl[i]; Yl[i] := Yl[i+n]; Yl[i+n] := j;
  j := Yr[i]; Yr[i] := Yr[i+n]; Yr[i+n] := j;
  j := ReadStart[i]; ReadStart[i] := ReadStart[i+n]; ReadStart[i+n] := j;
  j := ReadEnd[i]; ReadEnd[i] := ReadEnd[i+n]; ReadEnd[i+n] := j;
  j := LineStyle[i]; LineStyle[i] := LineStyle[i+n]; LineStyle[i+n] := j;
  j := LineColor[i]; LineColor[i] := LineColor[i+n]; LineColor[i+n] := j;
  j := MarkStyle[i]; MarkStyle[i] := MarkStyle[i+n]; MarkStyle[i+n] := j;
  j := MarkColor[i]; MarkColor[i] := MarkColor[i+n]; MarkColor[i+n] := j;
  j := Skip[i]; Skip[i] := Skip[i+n]; Skip[i+n] := j;
  x := Line_Width[i]; Line_Width[i] := Line_Width[i+n]; Line_Width[i+n] := x;
  x := MarkSize[i]; MarkSize[i] := MarkSize[i+n]; MarkSize[i+n] := x;
  x := Xmult[i]; Xmult[i] := Xmult[i+n]; Xmult[i+n] := x;
  x := Xoffset[i]; Xoffset[i] := Xoffset[i+n]; Xoffset[i+n] := x;
  x := Ymult[i]; Ymult[i] := Ymult[i+n]; Ymult[i+n] := x;
  x := Yoffset[i]; Yoffset[i] := Yoffset[i+n]; Yoffset[i+n] := x;
  S := XConverter[i]; XConverter[i] := XConverter[i+n]; XConverter[i+n] := S;
  S := YConverter[i]; YConverter[i] := YConverter[i+n]; YConverter[i+n] := S;
  j := XConvert[i]; XConvert[i] := XConvert[i+n]; XConvert[i+n] := j;
  j := YConvert[i]; YConvert[i] := YConvert[i+n]; YConvert[i+n] := j;
  j := NumData[i]; NumData[i] := NumData[i+n]; NumData[i+n] := j;
  j := CurveMode[i]; CurveMode[i] := CurveMode[i+n]; CurveMode[i+n] := j;
  j := ObjDataSeries[i]; ObjDataSeries[i] := ObjDataSeries[i+n]; ObjDataSeries[i+n] := j;
  j := DLcol[i]; DLcol[i] := DLcol[i+n]; DLcol[i+n] := j;
  j := DLsize[i]; DLsize[i] := DLsize[i+n]; DLsize[i+n] := j;
  x := Precision[i]; Precision[i] := Precision[i+n]; Precision[i+n] := x;
  S := CXmax[i]; CXmax[i] := CXmax[i+n]; CXmax[i+n] := S;
  S := CXmin[i]; CXmin[i] := CXmin[i+n]; CXmin[i+n] := S;
  B := PaintBelow[i]; PaintBelow[i] := PaintBelow[i+n]; PaintBelow[i+n] := B;
  if i + n = Target_of_integral then
    Target_of_integral := i;
  for i2 := 1 to 8 do
  begin
    x := P[i,i2]; P[i,i2] := P[i+n,i2]; P[i+n,i2] := x;
    B := Pfix[i,i2]; Pfix[i,i2] := Pfix[i+n,i2]; Pfix[i+n,i2] := B;
  end;
  for i2 := 1 to NumData[i+n] do
  begin
    tempx[i2] := GX[i,i2];
    tempy[i2] := GY[i,i2];
  end;
  for i2 := 1 to NumData[i] do
  begin
    GX[i,i2] := GX[i+n,i2];
    GY[i,i2] := GY[i+n,i2];
  end;
  for i2 := 1 to NumData[i+n] do
  begin
    GX[i+n,i2] := tempx[i2];
    GY[i+n,i2] := tempy[i2];
  end;
  //-------------------------
  for i2 := 6 to NumLegend do
  begin
    Legend[i2] := StringReplace(Legend[i2], '%' + IntToStr(i), 'tempstring', []);
    Legend[i2] := StringReplace(Legend[i2], '%' + IntToStr(i+n), '%' + IntToStr(i), []);
    Legend[i2] := StringReplace(Legend[i2], 'tempstring', '%' + IntToStr(i+n), []);
  end;
  Main.ActivateAxes;
  Modified[NumFigure] := 1;
  FormShow(Self);
  ListBox.ItemIndex := i - Direction - 1;
  ListBoxClick(Self);
end;
//----------------------------------------------------------
procedure  TDSlist.DrawLegendIndicator;
var i, y, Xm, Ym, lm: Integer;
begin
  StyleLine.Canvas := Image1.Canvas;
  with Image1.Canvas do
  begin
    Brush.Color := clWhite;
    Rectangle( -1, -1, Width + 1, Height + 1);
  end;
  if (CurveMode[DS] <> 0) and (ObjectSelecting = True) then
    dumint := (NumDataSeries - NumSplineCurve) - 1
  else
    dumint := (NumDataSeries - NumSplineCurve);
  for i := 1 to dumint do
  begin
    y := 14 * i - 6;
    if LineStyle[i] <> 0 then
    begin
      Main.SetLineProperty(Image1.Canvas, StyleLine, a0, i);
      with StyleLine do
      begin
        MoveTo(2, y);
        LineTo(Image1.Width - 2, y);
      end;
    end;
    if MarkStyle[i] <> 0 then
    begin
      with Image1.Canvas do
      begin
        Pen.Width := 1;
        Brush.Color := MarkColor[i];
        Brush.Style := bsSolid;
        Pen.Color := clBlack;
        Xm := Image1.Width div 2;
        Ym := y;
        lm := Ceil(MarkSize[i] / 2 * asc);
        Main.PlotMark(Image1.Canvas, Xm, Ym ,lm, i);
{
        Case MarkStyle[i] of
        1 : Ellipse(Xm - lm , Ym - lm, Xm + lm , Ym + lm);
        2 : Rectangle(Xm - lm , Ym - lm, Xm + lm , Ym + lm);
        3 : begin
            lm := lm * 2;
            Polygon([Point(Xm, Ym - Ceil(lm * 2 / 3)), Point(Xm + Ceil(lm / 2), Ym + Ceil(lm / 3)), Point(Xm - Ceil(lm / 2), Ym + Ceil(lm / 3))]);
            end;
        4 : begin
            lm := lm * 2;
            Polygon([Point(Xm, Ym + Ceil(lm * 2 / 3)), Point(Xm + Ceil(lm / 2), Ym - Ceil(lm / 3)), Point(Xm - Ceil(lm / 2), Ym - Ceil(lm / 3))]);
            end;
        5 : begin
            lm := Trunc(lm * 1.3);
            Polygon([Point(Xm, Ym - lm), Point(Xm + lm, Ym), Point(Xm, Ym + lm), Point(Xm - lm, Ym)]);
            end;
        6 : begin
            Brush.Color := MarkColor[i];
            Pen.Color   := Brush.Color;
            Polygon([Point(Xm, Ym + lm), Point(Xm + lm, Ym), Point(Xm, Ym)]);
            Brush.Color := Main.GetLighterColor(MarkColor[i], 0.6);
            Pen.Color   := Brush.Color;
            Polygon([Point(Xm, Ym - lm), Point(Xm + lm, Ym), Point(Xm, Ym)]);
            Brush.Color := Main.GetLighterColor(MarkColor[i], 0.6);
            Pen.Color   := Brush.Color;
            Polygon([Point(Xm, Ym + lm), Point(Xm - lm, Ym), Point(Xm, Ym)]);
            Brush.Color := Main.GetLighterColor(MarkColor[i], 0.2);
            Pen.Color   := Brush.Color;
            Polygon([Point(Xm, Ym - lm), Point(Xm - lm, Ym), Point(Xm, Ym)]);
            end;
        7 : begin
            Brush.Color := MarkColor[i];
            Pen.Color   := Brush.Color;
            Ellipse(Xm - lm , Ym - lm, Xm + lm , Ym + lm);
            Brush.Color := Main.GetLighterColor(MarkColor[i], 0.5);
            Pen.Color   := Brush.Color;
            Ellipse(Xm - Ceil(lm * 3 / 4), Ym - Ceil(lm * 3 / 4), Xm + Ceil(lm * 1 / 2) , Ym + Ceil(lm * 1 / 2));
            Brush.Color := Main.GetLighterColor(MarkColor[i], 0.1);
            Pen.Color   := Brush.Color;
            Ellipse(Xm - Ceil(lm * 4 / 7), Ym - Ceil(lm * 4 / 7), Xm + Ceil(lm * 1 / 7) , Ym + Ceil(lm * 1 / 7)) ;
            end;
        end;
 }
        Brush.Color := clWhite;
      end;
    end;
  end;
end;
//----------------------------------------------------------

procedure TDSlist.btnReadValueClick(Sender: TObject);
begin
  dumint := 219210;
  DS := ListBox.ItemIndex + 1;
  Close;
end;

procedure TDSlist.ListBoxKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  {$ifdef Linux}
  if Key = Key_Escape then
  {$else}
  if Key = VK_ESCAPE then
  {$endif}
    Close;
  ListBoxClick(Sender);
end;

procedure TDSlist.ListBoxClick(Sender: TObject);
begin
  if ListBox.ItemIndex > 0 then
    btnGoUp.Enabled := True
  else
    btnGoUp.Enabled := False;
  if ListBox.ItemIndex < NumDataSeries - NumSplineCurve - 1 then
    btnGoDown.Enabled := True
  else
    btnGoDown.Enabled := False;
end;

procedure TDSlist.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SelectedDS := 1;
end;

procedure TDSlist.Timer1Timer(Sender: TObject);
begin
  AutoSelect := False;
  Timer1.Enabled := False;
  btnEditClick(Sender);
end;

end.
