unit DataSeries_Dialog;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, QComCtrls, 
  MathsParser, QGrids, Types, QTypes, Qt
  , IniFiles, ExtCtrls, Dialogs, Grids, StdCtrls, Spin, Controls;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Spin, MathsParser, Grids, IniFiles;
{$endif}

const
  NumRow = 78;

type
  TDataSeriesDialog = class(TForm)
    cbXb: TComboBox;
    cbXt: TComboBox;
    cbYl: TComboBox;
    cbYr: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label7: TLabel;
    cbReadEnd: TComboBox;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label10: TLabel;
    Label11: TLabel;
    cbLizeStyle: TComboBox;
    Label12: TLabel;
    edLineWidth: TEdit;
    GroupBox3: TGroupBox;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    cbMarkStyle: TComboBox;
    edMarkSize: TEdit;
    Panel1: TPanel;
    Label16: TLabel;
    Panel2: TPanel;
    OK: TButton;
    CancelButton: TButton;
    Panel3: TPanel;
    ColorDialog1: TColorDialog;
    Image1: TImage;
    Timer1: TTimer;
    Button1: TButton;
    Panel4: TPanel;
    Image2: TImage;
    Button2: TButton;
    ColorDialog2: TColorDialog;
    chkSpline: TCheckBox;
    GroupBox4: TGroupBox;
    Label17: TLabel;
    Label18: TLabel;
    seLabelSize: TSpinEdit;
    GroupBox5: TGroupBox;
    GroupBox6: TGroupBox;
    GroupBox7: TGroupBox;
    Label1: TLabel;
    chkXconvert: TCheckBox;
    chkYconvert: TCheckBox;
    edXconverter: TEdit;
    edYconverter: TEdit;
    Parser: TMathsParser;
    StringGrid: TStringGrid;
    cbErrBarXlow: TComboBox;
    cbErrBarYlow: TComboBox;
    cbDataLabels: TComboBox;
    chkPaintBelow: TCheckBox;
    Panel5: TPanel;
    Label6: TLabel;
    Panel6: TPanel;
    Label8: TLabel;
    Panel7: TPanel;
    Label9: TLabel;
    Panel8: TPanel;
    Label19: TLabel;
    cbErrBarXhigh: TComboBox;
    cbErrBarYhigh: TComboBox;
    HelpButton: TButton;
    Label20: TLabel;
    seSkip: TSpinEdit;
    edReadStart: TEdit;
    Label21: TLabel;
    chk3D: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure cbXbChange(Sender: TObject);
    procedure cbXtChange(Sender: TObject);
    procedure cbYlChange(Sender: TObject);
    procedure cbYrChange(Sender: TObject);
    procedure OKClick(Sender: TObject);
    procedure cbReadEndKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Panel3Enter(Sender: TObject);
    procedure Panel3Exit(Sender: TObject);
    procedure Panel3MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Timer1Timer(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Panel4Enter(Sender: TObject);
    procedure Panel4Exit(Sender: TObject);
    procedure Panel4MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image2MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure chkXconvertClick(Sender: TObject);
    procedure chkYconvertClick(Sender: TObject);
    procedure StringGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure cbDataLabelsChange(Sender: TObject);
    procedure StringGridExit(Sender: TObject);
    procedure StringGridEnter(Sender: TObject);
    procedure cbErrBarXlowChange(Sender: TObject);
    procedure cbErrBarXhighChange(Sender: TObject);
    procedure cbErrBarYlowChange(Sender: TObject);
    procedure cbErrBarYhighChange(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
    function IsNumeral(S: String): Boolean;
    function DefColor: Integer;
  public
    function SetLengthRJ(dumstr: String; dumint: Shortint): String;
  end;

var
  DataSeriesDialog: TDataSeriesDialog;
  SL1, SL2: TStringList;
  Msg: array[1..10] of String;
  MaxColWidth: array[1..NumRow] of Integer;
  LastIndex_of_cbXb, LastIndex_of_cbXt, LastIndex_of_cbYl,  LastIndex_of_cbYr,
  LastIndex_of_cbErrBarXlow, LastIndex_of_cbErrBarYlow, LastIndex_of_cbDataLabels, LastIndex_of_cbErrBarXhigh, LastIndex_of_cbErrBarYhigh, NumItemsCB5: ShortInt;

implementation

uses Grapher_Main, DS_list;

{$R *.xfm}

procedure TDataSeriesDialog.FormCreate(Sender: TObject);
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
  i: Integer;
  procedure GetComboBoxItems(CB: TComboBox );
  begin
    i := CB.ItemIndex;
    CB.Items.Text := StringReplace(IniFile.ReadString(S, CB.Name +  '.Items.Text', ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
    CB.ItemIndex := i;
  end;
begin
  Msg[1] := 'Properties of Data Series - ';
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  Font.Assign(Main.Font);
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        GroupBox5.Caption        := ReadString(S, 'GroupBox5.Caption', '');
        GroupBox6.Caption        := ReadString(S, 'GroupBox6.Caption', '');
        Label7.Caption           := ReadString(S, 'Label7.Caption', '');
        Label20.Caption          := ReadString(S, 'Label20.Caption', '');
        Label21.Caption          := ReadString(S, 'Label21.Caption', '');
        GroupBox1.Caption        := ReadString(S, 'GroupBox1.Caption', '');
        chkXconvert.Caption      := ReadString(S, 'chkXconvert.Caption', '');
        chkYconvert.Caption      := ReadString(S, 'chkYconvert.Caption', '');
        GroupBox2.Caption        := ReadString(S, 'GroupBox2.Caption', '');
        Label10.Caption          := ReadString(S, 'Label10.Caption', '');
        Label11.Caption          := ReadString(S, 'Label11.Caption', '');
        Label12.Caption          := ReadString(S, 'Label12.Caption', '');
        chkSpline.Caption        := ReadString(S, 'chkSpline.Caption', '');
        chkPaintBelow.Caption    := ReadString(S, 'chkPaintBelow.Caption', '');
        GroupBox3.Caption        := ReadString(S, 'GroupBox3.Caption', '');
        Label13.Caption          := ReadString(S, 'Label13.Caption', '');
        Label14.Caption          := ReadString(S, 'Label14.Caption', '');
        Label15.Caption          := ReadString(S, 'Label15.Caption', '');
        CancelButton.Caption     := ReadString(S, 'CancelButton.Caption', '');
        HelpButton.Caption       := ReadString(S, 'HelpButton.Caption', '');
        GroupBox4.Caption        := ReadString(S, 'GroupBox4.Caption', '');
        Label17.Caption          := ReadString(S, 'Label17.Caption', '');
        Label18.Caption          := ReadString(S, 'Label18.Caption', '');
        GroupBox7.Caption        := ReadString(S, 'GroupBox7.Caption', '');
        Label1.Caption           := ReadString(S, 'Label1.Caption', '');
        Label16.Caption     := StringReplace(ReadString(S, 'Label16.Caption', ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
        cbReadEnd.Items.Strings[0] := ReadString(S, 'cbReadEnd.Items.Strings[0]', '');
        GetComboBoxItems(cbLizeStyle);
        GetComboBoxItems(cbMarkStyle);
        for i := 1 to 1 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  if LANG = 128 then
  begin
    edReadStart.Left := Label7.Left;
    Label7.Left := Label7.Left + 35;
    cbReadEnd.Left := 90;
    Label12.Left := 195;
    chkSpline.Left := 150;
    Label15.Left := 170;
    edMarkSize.Left := 215;
    Label21.Left := cbReadEnd.Left + cbReadEnd.Width;
  end;
  LastIndex_of_cbErrBarXlow := 0;
  LastIndex_of_cbErrBarYlow := 0;
  LastIndex_of_cbErrBarXhigh := 0;
  LastIndex_of_cbErrBarYhigh := 0;
  LastIndex_of_cbDataLabels := 0;
  NumItemsCB5 := 0;
  SelectedDS := 1;
end;

procedure TDataSeriesDialog.FormShow(Sender: TObject);
var
  i, j, j2, k: Integer;
  S: String;
  SearchRec: TSearchRec;
begin
  ModalResult := mrNone;
  i := DS;
  if FindFirst(DataSeries[i], faAnyFile, SearchRec) = 0 then
    S := DateTimeToStr(FileDateToDateTime((SearchRec.Time)));
  Caption := Msg[1] + Main.GetRelativeDirectory(FigDir, DataSeries[i]) + ': ' + Copy(S, 0, Length(S) - 3);
  FindFirst(DataSeries[i], faAnyFile, SearchRec);
  FindClose(SearchRec);
  Main.Cursor := crDefault;
  cbXb.ItemIndex := Xb[i];
  cbXt.ItemIndex := Xt[i];
  cbYl.ItemIndex := Yl[i];
  cbYr.ItemIndex := Yr[i];
  LastIndex_of_cbXb := cbXb.ItemIndex;
  LastIndex_of_cbXt := cbXt.ItemIndex;
  LastIndex_of_cbYl := cbYl.ItemIndex;
  LastIndex_of_cbYr := cbYr.ItemIndex;
  cbXbChange(Sender);
  cbXtChange(Sender);
  cbYlChange(Sender);
  cbYrChange(Sender);
  edReadStart.Text := IntToStr(ReadStart[i]);
  if ReadEnd[i] = 0 then
    cbReadEnd.ItemIndex := 0
  else
  begin
    if cbReadEnd.Items.IndexOf(IntToStr(ReadEnd[i])) = -1 then
    begin
      NumItemsCB5 := NumItemsCB5 + 1;
      cbReadEnd.Items.Add(IntToStr(ReadEnd[i]));
      cbReadEnd.ItemIndex := NumItemsCB5;
    end
    else
      cbReadEnd.ItemIndex := cbReadEnd.Items.IndexOf(IntToStr(ReadEnd[i]));
  end;
  LineStyle[i] := ((LineStyle[i] + 1) mod 6) - 1;
  cbLizeStyle.ItemIndex := LineStyle[i];
  if Spline[i] = True then
    chkSpline.Checked := True
  else
    chkSpline.Checked := False;
  if PaintBelow[i] = True then
    chkPaintBelow.Checked := True
  else
    chkPaintBelow.Checked := False;
  if MarkStyle[i] > 10 then
  begin
    chk3D.Checked := True;
    cbMarkStyle.ItemIndex := MarkStyle[i] - 10;
  end
  else
  begin
    chk3D.Checked := False;
    cbMarkStyle.ItemIndex := MarkStyle[i];
  end;
  edLineWidth.Text := FloatToStr(Round(10 * Line_Width[i]) / 10);
  edMarkSize.Text := FloatToStr(MarkSize[i]);
  seSkip.Value := Skip[i];
  {
  edXconverter.Text := FloatToStr(Xmult[i]);
  Memo3.Lines[0] := FloatToStr(Xoffset[i]);
  edYconverter.Text := FloatToStr(Ymult[i]);
  Memo5.Lines[0] := FloatToStr(Yoffset[i]);
  }
  edXconverter.Text := XConverter[i];
  edYconverter.Text := YConverter[i];
  chkXconvertClick(Sender);
  chkYconvertClick(Sender);
  if XConvert[i] = 0 then chkXconvert.Checked := False else chkXconvert.Checked := True;
  if YConvert[i] = 0 then chkYconvert.Checked := False else chkYconvert.Checked := True;
  Panel3.Color := LineColor[i];
  Panel3Exit(Sender);
  Panel4.Color := MarkColor[i];
  Panel4Exit(Sender);
  cbDataLabels.ItemIndex := DLcol[i];
  seLabelSize.Value := DLsize[i];
  cbErrBarXlow.ItemIndex := Xerlow[i];
  cbErrBarXlowChange(Sender);
  cbErrBarYlow.ItemIndex := Yerlow[i];
  cbErrBarYlowChange(Sender);
  cbErrBarXhigh.ItemIndex := Xerhigh[i];
  cbErrBarXhighChange(Sender);
  cbErrBarYhigh.ItemIndex := Yerhigh[i];
  cbErrBarYhighChange(Sender);
  {
  if NumLoadedData > 1 then
    cbDataLabels.Enabled := False
  else
    cbDataLabels.Enabled := True;
  }
  cbDataLabelsChange(Sender);
  //--------------------
  for j := 1 to 200 do
    for k := 1 to NumRow do
      StringGrid.Cells[k,j] := '';
  SL1 := TStringList.Create;
  SL2 := TStringList.Create;
  i := DS;
  for j := 1 to NumRow do
    MaxColWidth[j] := 30;
  try
    {$ifdef Linux}
    if Main.mmJconvert.Checked then
    begin
      Main.LibcSystem('nkf -e "' + DataSeries[i] + '" > ' + AppPath + 'temp.txt');
      SL1.LoadFromFile(AppPath + 'temp.txt');
      DeleteFile(AppPath + 'temp.txt');
    end
    else
      SL1.LoadFromFile(DataSeries[i]);
    {$else}
    SL1.LoadFromFile(DataSeries[i]);
    {$endif}
    for i := 0 to SL1.Count - 1 do
    begin
      SL2.CommaText := SL1[i];
      for j := SL2.Count - 1 downto 0 do
      begin
        StringGrid.Cells[j+1,i+1] := SL2[j];
        //-- Detecting blank area (in the text line) which should be blank cell --//
        if j > 0 then
        begin
          if Pos(SL2[j], SL1[i]) > Pos(SL2[j-1], SL1[i]) + Length(SL2[j-1]) + 12 then
            if IsNumeral(SL2[j]) = False then
              if IsNumeral(SL2[j-1]) = False then
              begin
                for j2 := NumRow downto j + 3 do
                  StringGrid.Cells[j2,i+1] := StringGrid.Cells[j2-1,i+1];
                StringGrid.Cells[j+2,i+1] := SL2[j];
                StringGrid.Cells[j+1,i+1] := '';
              end;
        end
        else
        begin
          if Pos(SL2[0], SL1[i]) > 10 then
            if IsNumeral(SL2[0]) = False then
              begin
                for j2 := NumRow downto 3 do
                  StringGrid.Cells[j2,i+1] := StringGrid.Cells[j2-1,i+1];
                StringGrid.Cells[2,i+1] := SL2[0];
                StringGrid.Cells[1,i+1] := '';
              end;
        end;
      end;
      for j := 0 to 25 do
        if (StringGrid.Font.Size * Length(StringGrid.Cells[j+1,i+1])) div 2 + 20 > MaxColWidth[j+1] then
          MaxColWidth[j+1] := 30 + 20 + (StringGrid.Font.Size * Length(StringGrid.Cells[j+1,i+1])) div 2;
    end;
    StringGrid.RowCount := SL1.Count + 1;
  finally
    SL1.Free;
    SL2.Free;
  end;
  with StringGrid do
  begin
    DefaultDrawing := True;
    ColWidths[0] := 30;
    for i := 1 to RowCount + 1 do
    begin
      S := IntToStr(i);
      for j := 1 to 5 - Length(S) do
        S :=  ' '  + S;
      Cells[0,i] := S;
    end;
    SetFocus;
  end;
  for j := 1 to NumRow do
  begin
    StringGrid.ColWidths[j] := MaxColWidth[j];
    if j <= 26 then
    begin
      S := Char(64 + j);
      k := 0;
    end
    else
      if j <= 52 then
      begin
        S := 'A' + Char(64 + j - 26);
        k := -1;
      end
      else
      begin
        S := 'B' + Char(64 + j - 52);
        k := -1;
      end;
    for i := 1 to (k + StringGrid.ColWidths[j] div StringGrid.Font.Size) do
      S := ' ' + S;
   // StringGridDrawCell(Sender, 2, 2, );
    StringGrid.Cells[j,0] := S;
  end;
  if cbXb.ItemIndex <> 0 then
    cbXb.Color := 11599792
  else
    cbXb.Color := clWhite;
  if cbXt.ItemIndex <> 0 then
    cbXt.Color := 11599792
  else
    cbXt.Color := clWhite;
  if cbYl.ItemIndex <> 0 then
    cbYl.Color := 16777130
  else
    cbYl.Color := clWhite;
  if cbYr.ItemIndex <> 0 then
    cbYr.Color := 16777130
  else
    cbYr.Color := clWhite;
  if cbDataLabels.ItemIndex <> 0 then
    cbDataLabels.Color := 15395562
  else
    cbDataLabels.Color := clWhite;
  //StringGridEnter(Sender);
end;

function TDataSeriesDialog.IsNumeral(S: String): Boolean;
var d: Double;
begin
  try
    d := StrToFloat(S);
    if d > 0 then ;
    Result := True;
  except
    Result := False;
  end;
end;

procedure TDataSeriesDialog.StringGridDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  {$ifdef Linux}
  i: Integer;
  {$endif}
  DoIt: Boolean;
begin
  DoIt := False;
  {$ifdef Linux}
  i := StrToInt(edReadStart.Text);
  if (ACol > 0) and (ARow > 0) and (ARow < i + 7) then
  begin
  {$endif}
    if gdFocused in State then
      StringGrid.Canvas.DrawFocusRect(Rect);
    if (ACol > 0) and (ARow > 0) then
      with StringGrid.Canvas do
      begin
        Brush.Color := clWhite;
        DoIt := True;
      end;
    if (cbXb.ItemIndex > 0) and (ACol = cbXb.ItemIndex) and (ARow > 0) then
      with StringGrid.Canvas do
      begin
        Brush.Color := 11599792;
        DoIt := True;
      end;
    if (cbXt.ItemIndex > 0) and (ACol = cbXt.ItemIndex) and (ARow > 0) then
      with StringGrid.Canvas do
      begin
        Brush.Color := 11599792;
        DoIt := True;
      end;
    if (cbYl.ItemIndex > 0) and (ACol = cbYl.ItemIndex) and (ARow > 0) then
      with StringGrid.Canvas do
      begin
        Brush.Color := 16777130;
        DoIt := True;
      end;
    if (cbYr.ItemIndex > 0) and (ACol = cbYr.ItemIndex) and (ARow > 0) then
      with StringGrid.Canvas do
      begin
        Brush.Color := 16777130;
        DoIt := True;
      end;
    if (cbErrBarXlow.ItemIndex > 0) and (ACol = cbErrBarXlow.ItemIndex) and (ARow > 0) then
      with StringGrid.Canvas do
      begin
        Brush.Color := 11586303;
        DoIt := True;
      end;
    if (cbErrBarYlow.ItemIndex > 0) and (ACol = cbErrBarYlow.ItemIndex) and (ARow > 0) then
      with StringGrid.Canvas do
      begin
        Brush.Color := 16764663;
        DoIt := True;
      end;
    if (cbErrBarXlow.ItemIndex > 0) and (ACol = cbErrBarXlow.ItemIndex) and (ARow > 0) then
      with StringGrid.Canvas do
      begin
        Brush.Color := 11586303;
        DoIt := True;
      end;
    if (cbErrBarYlow.ItemIndex > 0) and (ACol = cbErrBarYlow.ItemIndex) and (ARow > 0) then
      with StringGrid.Canvas do
      begin
        Brush.Color := 16764663;
        DoIt := True;
      end;
    if (cbDataLabels.ItemIndex > 0) and (ACol = cbDataLabels.ItemIndex) and (ARow > 0) then
      with StringGrid.Canvas do
      begin
        Brush.Color := 15395562;
        DoIt := True;
      end;
    if DoIt then
      with StringGrid.Canvas do
      begin
        FillRect(Rect);
        try
          dum := StrToFloat(StringGrid.Cells[ACol,ARow]);
          {$ifdef MSWINDOWS}
          DrawText(Handle, PChar(StringGrid.Cells[ACol,ARow]), Length(StringGrid.Cells[ACol,ARow]), Rect, DT_RIGHT);
          {$else}
          TextOut(Rect.Left+2,Rect.Top+2,StringGrid.Cells[ACol,ARow]);
          {$endif}
        except
          TextOut(Rect.Left+2,Rect.Top+2,StringGrid.Cells[ACol,ARow]);
        end;
      end;
  {$ifdef Linux}
  end;
  {$endif}
end;

//----------------------------------------------------------

procedure TDataSeriesDialog.cbXbChange(Sender: TObject);
var j: Integer;
begin
  if cbXb.ItemIndex <> 0 then
  begin
    cbXt.ItemIndex := 0;
    cbXtChange(Sender);
    cbXb.Color := 11599792 //11599792
  end
  else
    cbXb.Color := clWhite;
  if (cbYl.ItemIndex = cbXb.ItemIndex) and (cbYl.ItemIndex > 0) then
  begin
    if cbXb.ItemIndex = 2 then
      cbYl.ItemIndex := 1
    else
      cbYl.ItemIndex := cbXb.ItemIndex + 1;
    cbYlChange(Sender);
  end;
  if (cbYr.ItemIndex = cbXb.ItemIndex) and (cbYr.ItemIndex > 0) then
  begin
    if cbXb.ItemIndex = 2 then
      cbYr.ItemIndex := 1
    else
      cbYr.ItemIndex := cbXb.ItemIndex + 1;
    cbYrChange(Sender);
  end;
  for j := 1 to StringGrid.RowCount do
  begin
    if cbXb.ItemIndex < 53 then
      StringGrid.Cells[cbXb.ItemIndex,j]  := StringGrid.Cells[cbXb.ItemIndex,j];
    StringGrid.Cells[LastIndex_of_cbXb,j] := StringGrid.Cells[LastIndex_of_cbXb,j];
  end;
  if cbXb.ItemIndex < 53 then
    LastIndex_of_cbXb := cbXb.ItemIndex;
end;

procedure TDataSeriesDialog.cbXtChange(Sender: TObject);
var j: Integer;
begin
  if cbXt.ItemIndex <> 0 then
  begin
    cbXb.ItemIndex := 0;
    cbXbChange(Sender);
    chkPaintBelow.Checked := False;
    chkPaintBelow.Enabled := False;
    cbXt.Color := 11599792 //11599792
  end
  else
  begin
    cbXt.Color := clWhite;
    chkPaintBelow.Enabled := True;
  end;
  if (cbYl.ItemIndex = cbXt.ItemIndex) and (cbYl.ItemIndex > 0) then
  begin
    if cbXt.ItemIndex = 2 then
      cbYl.ItemIndex := 1
    else
      cbYl.ItemIndex := cbXt.ItemIndex + 1;
    cbYlChange(Sender);
  end;
  if (cbYr.ItemIndex = cbXt.ItemIndex) and (cbYr.ItemIndex > 0) then
  begin
    if cbXt.ItemIndex = 2 then
      cbYr.ItemIndex := 1
    else
      cbYr.ItemIndex := cbXt.ItemIndex + 1;
    cbYrChange(Sender);
  end;
  for j := 1 to StringGrid.RowCount do
  begin
    if cbXb.ItemIndex < 53 then
      StringGrid.Cells[cbXt.ItemIndex,j]    := StringGrid.Cells[cbXt.ItemIndex,j];
    StringGrid.Cells[LastIndex_of_cbXt,j] := StringGrid.Cells[LastIndex_of_cbXt,j];
  end;
  if cbXt.ItemIndex < 53 then
    LastIndex_of_cbXt := cbXt.ItemIndex;
end;

procedure TDataSeriesDialog.cbYlChange(Sender: TObject);
var j: Integer;
begin
  if cbYl.ItemIndex <> 0 then
  begin
    cbYl.Color := 16777130; // 16777130
    cbYr.ItemIndex := 0;
    cbYrChange(Sender);
  end
  else
    cbYl.Color := clWhite;
  if (cbXb.ItemIndex = cbYl.ItemIndex) and (cbXb.ItemIndex > 0) then
  begin
    if cbYl.ItemIndex = 2 then
      cbXb.ItemIndex := 1
    else
      cbXb.ItemIndex := cbYl.ItemIndex + 1;
    cbXbChange(Sender);
  end;
  if (cbXt.ItemIndex = cbYl.ItemIndex) and (cbXt.ItemIndex > 0) then
  begin
    if cbYl.ItemIndex = 2 then
      cbXt.ItemIndex := 1
    else
      cbXt.ItemIndex := cbYl.ItemIndex + 1;
    cbXtChange(Sender);
  end;
  for j := 1 to StringGrid.RowCount do
  begin
    StringGrid.Cells[cbYl.ItemIndex,j]    := StringGrid.Cells[cbYl.ItemIndex,j];
    StringGrid.Cells[LastIndex_of_cbYl,j] := StringGrid.Cells[LastIndex_of_cbYl,j];
  end;
  LastIndex_of_cbYl := cbYl.ItemIndex;
end;

procedure TDataSeriesDialog.cbYrChange(Sender: TObject);
var j: Integer;
begin
  if cbYr.ItemIndex <> 0 then
  begin
    cbYr.Color := 16777130;
    cbYl.ItemIndex := 0;
    cbYlChange(Sender);
  end
  else
    cbYr.Color := clWhite;
  if (cbXb.ItemIndex = cbYr.ItemIndex) and (cbXb.ItemIndex > 0) then
  begin
    if cbYr.ItemIndex = 2 then
      cbXb.ItemIndex := 1
    else
      cbXb.ItemIndex := cbYr.ItemIndex + 1;
    cbXbChange(Sender);
  end;
  if (cbXt.ItemIndex = cbYr.ItemIndex) and (cbXt.ItemIndex > 0) then
  begin
    if cbYr.ItemIndex = 2 then
      cbXt.ItemIndex := 1
    else
      cbXt.ItemIndex := cbYr.ItemIndex + 1;
    cbXtChange(Sender);
  end;
  for j := 1 to StringGrid.RowCount do
  begin
    StringGrid.Cells[cbYr.ItemIndex,j]    := StringGrid.Cells[cbYr.ItemIndex,j];
    StringGrid.Cells[LastIndex_of_cbYr,j] := StringGrid.Cells[LastIndex_of_cbYr,j];
  end;
  LastIndex_of_cbYr := cbYr.ItemIndex;
end;

procedure TDataSeriesDialog.OKClick(Sender: TObject);
var
  i: Integer;
begin
  i := DS;
  Xb[i] := cbXb.ItemIndex;
  Xt[i] := cbXt.ItemIndex;
  Yl[i] := cbYl.ItemIndex;
  Yr[i] := cbYr.ItemIndex;
  ReadStart[i] := StrToInt(edReadStart.Text);
  try
    ReadEnd[i] := StrToInt(cbReadEnd.Text);
  except
    ReadEnd[i] := 0
  end;
  LineStyle[i] := cbLizeStyle.ItemIndex;
  if chkSpline.Checked then
    Spline[i] := True
  else
    Spline[i] := False;
  Line_Width[i] := StrToFloat(edLineWidth.Text);
  if chkPaintBelow.Checked then
    PaintBelow[i] := True
  else
    PaintBelow[i] := False;
  MarkStyle[i] := cbMarkStyle.ItemIndex;
  if chk3D.Checked then
    MarkStyle[i] := MarkStyle[i] + 10;
  MarkSize[i]  := StrToFloat(edMarkSize.Text);
  {
  if edXconverter.Text = ''  then edXconverter.Text := ' 1';
  if edXconverter.Text = '-' then edXconverter.Text := '-1';
  if edYconverter.Text = ''  then edYconverter.Text := ' 1';
  if edYconverter.Text = '-' then edYconverter.Text := '-1';
  Xmult[i]   := StrToFloat(edXconverter.Text);
  Xoffset[i] := StrToFloat(Memo3.Lines[0]);
  Ymult[i]   := StrToFloat(edYconverter.Text);
  Yoffset[i] := StrToFloat(Memo5.Lines[0]);
  }
  if Pos('x', AnsiLowerCase(edXconverter.Text)) > 0 then
    XConverter[i] := edXconverter.Text;
  if Pos('y', AnsiLowerCase(edYconverter.Text)) > 0 then
    YConverter[i] := edYconverter.Text;
  if chkXconvert.Checked then
  begin
    XConvert[i] := 1;
    Parser.Expression := XConverter[i];    // Validation
  end
  else
   XConvert[i] := 0;
  if chkYconvert.Checked then
  begin
    YConvert[i] := 1;
    Parser.Expression := YConverter[i];
  end
  else
    YConvert[i] := 0;
  DLcol[i] := cbDataLabels.ItemIndex;
  DLsize[i]:= seLabelSize.Value;
  Xerlow[i]   := cbErrBarXlow.ItemIndex;
  Yerlow[i]   := cbErrBarYlow.ItemIndex;
  Xerhigh[i]   := cbErrBarXhigh.ItemIndex;
  Yerhigh[i]   := cbErrBarYhigh.ItemIndex;
  Skip[i] := seSkip.Value;
  Main.LoadDataSeries;
  Modified[NumFigure] := 1;
  ModalResult := mrOK;
end;
//----------------------------------------------------------

procedure TDataSeriesDialog.cbReadEndKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  {$ifdef Linux}
  if key = Key_Return then
  {$else}
  if key = VK_RETURN then
  {$endif}
    if cbReadEnd.Items.IndexOf( cbReadEnd.Text ) = -1 then
    begin
      NumItemsCB5 := NumItemsCB5 + 1;
      cbReadEnd.Items.Add(cbReadEnd.Text);
      cbReadEnd.ItemIndex := NumItemsCB5;
    end;
end;
//----------------------------------------------------------
function TDataSeriesDialog.SetLengthRJ(dumstr: String; dumint: Shortint): String;
var i: Shortint;
begin
  for i:=1 to dumint - Length(dumstr) do
    dumstr:=' '+ dumstr;
  Result := dumstr;
end;
//----------------------------------------------------------

procedure TDataSeriesDialog.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  //if (ModalResult <> mrOK) and (ModalResult <> mrCancel) then
  //  SysUtils.Abort;
end;
//----------------------------------------------------------
procedure TDataSeriesDialog.Panel3Enter(Sender: TObject);
begin
  With Image1.Canvas do
  begin
    Pen.Color := clYellow;
    Pen.Style := psDot;
    Brush.Color := Panel3.Color;
    Rectangle(0, 0, Image1.Width, Image1.Height);
    OK.Default := False;
  end;
end;

procedure TDataSeriesDialog.Panel3Exit(Sender: TObject);
begin
  With Image1.Canvas do
  begin
    Pen.Color := Panel3.Color;
    Brush.Color := Panel3.Color;
    Rectangle(0, 0, Image1.Width, Image1.Height);
    OK.Default := True;
  end;
end;

procedure TDataSeriesDialog.Panel3MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  Panel3Enter(Sender);
  Timer1.Enabled := True;
end;

procedure TDataSeriesDialog.Image1MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  Button1Click(Sender)
end;

function TDataSeriesDialog.DefColor: Integer;
begin
  Case DS of
   1:Result := 0;        // black
   2:Result := 16711680; // blue
   3:Result := 33023;    // orange
   4:Result := 65280;    // lime
   5:Result := 255;      // red
   6:Result := 32896;    // olive
   7:Result := 65535;    // yellow
   8:Result := 32768;    // green
   9:Result := 16744576; // purple
  10:Result := 16711808; // aqua
  11:Result := 0;        // black
  12:Result := 16711680; // blue
  13:Result := 33023;    // orange
  14:Result := 65280;    // lime
  15:Result := 255;      // red
  16:Result := 32896;    // olive
  17:Result := 65535;    // yellow
  18:Result := 32768;    // green
  19:Result := 16744576; // purple
  20:Result := 16711808; // aqua
  else
    Result := 0;
  end;
end;

procedure TDataSeriesDialog.Button1Click(Sender: TObject);
begin
  ColorDialog1.Color :=  LineColor[DS];
  if ColorDialog1.Execute then
  begin
    LineColor[DS] := ColorDialog1.Color;
    Panel3.Color := ColorDialog1.Color;
    if MarkColor[DS] = DefColor then
    begin
      MarkColor[DS] := LineColor[DS];
      Panel4.Color  := MarkColor[DS];
      Panel4Exit(Sender);
    end;
  end;
 // ShowMessage(IntToStr(Panel3.Color));
  Panel3Exit(Sender);
end;

procedure TDataSeriesDialog.Panel4Enter(Sender: TObject);
begin
  With Image2.Canvas do
  begin
    Pen.Color := clYellow;
    Pen.Style := psDot;
    Brush.Color := Panel4.Color;
    Rectangle(0, 0, Image2.Width, Image2.Height);
    OK.Default := False;
  end;
end;

procedure TDataSeriesDialog.Panel4Exit(Sender: TObject);
begin
  With Image2.Canvas do
  begin
    Pen.Color := Panel4.Color;
    Brush.Color := Panel4.Color;
    Rectangle(0, 0, Image2.Width, Image2.Height);
    OK.Default := True;
  end;
end;

procedure TDataSeriesDialog.Panel4MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  Panel4Enter(Sender);
  Timer1.Enabled := True;
end;

procedure TDataSeriesDialog.Image2MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  Button2Click(Sender);
end;

procedure TDataSeriesDialog.Button2Click(Sender: TObject);
begin
  ColorDialog2.Color :=  MarkColor[DS];
  if ColorDialog2.Execute then
  begin
    MarkColor[DS] := ColorDialog2.Color;
    Panel4.Color := ColorDialog2.Color;
    if LineColor[DS] = DefColor then
    begin
      LineColor[DS] := MarkColor[DS];
      Panel3.Color  := MarkColor[DS];
      Panel3Exit(Sender);
    end;
  end;
  if Panel4.Color = clWhite then
  begin
    chk3D.Checked := False;
    chk3D.Enabled := False;
  end
  else
    chk3D.Enabled := True;
  Panel4Exit(Sender);
end;

procedure TDataSeriesDialog.Timer1Timer(Sender: TObject);
begin
  Timer1.Enabled := False;
  Panel3Exit(Sender);
  Panel4Exit(Sender);
end;

procedure TDataSeriesDialog.chkXconvertClick(Sender: TObject);
begin
  if chkXconvert.Checked then edXconverter.Enabled := True else edXconverter.Enabled := False;
end;

procedure TDataSeriesDialog.chkYconvertClick(Sender: TObject);
begin
  if chkYconvert.Checked then edYconverter.Enabled := True  else edYconverter.Enabled := False;
end;

procedure TDataSeriesDialog.cbDataLabelsChange(Sender: TObject);
var j: Integer;
begin
  if cbDataLabels.ItemIndex > 0 then
    seLabelSize.Enabled := True
  else
    seLabelSize.Enabled := False;
  {
  if NumLoadedData > 1 then
  begin
    cbDataLabels.Enabled := False;
    SpinedReadStart.Enabled := False;
  end;
  }
  if cbDataLabels.ItemIndex > 0 then
    cbDataLabels.Color := 15395562
  else
    cbDataLabels.Color := clWhite;
  for j := 1 to StringGrid.RowCount do
  begin
    StringGrid.Cells[cbDataLabels.ItemIndex,j]    := StringGrid.Cells[cbDataLabels.ItemIndex,j];
    StringGrid.Cells[LastIndex_of_cbDataLabels,j] := StringGrid.Cells[LastIndex_of_cbDataLabels,j];
  end;
  LastIndex_of_cbDataLabels := cbDataLabels.ItemIndex;
end;

procedure TDataSeriesDialog.cbErrBarXlowChange(Sender: TObject);
var j: Integer;
begin
  if cbErrBarXlow.ItemIndex > 0 then
    cbErrBarXlow.Color := 11586303
  else
    cbErrBarXlow.Color := clWhite;
  if (cbErrBarXlow.ItemIndex > 0) and (cbErrBarXhigh.ItemIndex = 0) then
  begin
    cbErrBarXhigh.ItemIndex := cbErrBarXlow.ItemIndex;
    cbErrBarXhighChange(Sender);
  end;
  for j := 1 to StringGrid.RowCount do
  begin
    StringGrid.Cells[cbErrBarXlow.ItemIndex,j]    := StringGrid.Cells[cbErrBarXlow.ItemIndex,j];
    StringGrid.Cells[LastIndex_of_cbErrBarXlow,j] := StringGrid.Cells[LastIndex_of_cbErrBarXlow,j];
  end;
  LastIndex_of_cbErrBarXlow := cbErrBarXlow.ItemIndex;
end;

procedure TDataSeriesDialog.cbErrBarXhighChange(Sender: TObject);
var j: Integer;
begin
  if cbErrBarXhigh.ItemIndex > 0 then
    cbErrBarXhigh.Color := 11586303
  else
    cbErrBarXhigh.Color := clWhite;
  if (cbErrBarXlow.ItemIndex = 0) and (cbErrBarXhigh.ItemIndex > 0) then
  begin
    cbErrBarXlow.ItemIndex := cbErrBarXhigh.ItemIndex;
    cbErrBarXlowChange(Sender);
  end;
  for j := 1 to StringGrid.RowCount do
  begin
    StringGrid.Cells[cbErrBarXhigh.ItemIndex,j]    := StringGrid.Cells[cbErrBarXhigh.ItemIndex,j];
    StringGrid.Cells[LastIndex_of_cbErrBarXhigh,j] := StringGrid.Cells[LastIndex_of_cbErrBarXhigh,j];
  end;
  LastIndex_of_cbErrBarXhigh := cbErrBarXhigh.ItemIndex;
end;

procedure TDataSeriesDialog.cbErrBarYlowChange(Sender: TObject);
var j: Integer;
begin
  if cbErrBarYlow.ItemIndex > 0 then
    cbErrBarYlow.Color := 16764663
  else
    cbErrBarYlow.Color := clWhite;
  if (cbErrBarYlow.ItemIndex > 0) and (cbErrBarYhigh.ItemIndex = 0) then
  begin
    cbErrBarYhigh.ItemIndex := cbErrBarYlow.ItemIndex;
    cbErrBarYhighChange(Sender);
  end;
  for j := 1 to StringGrid.RowCount do
  begin
    StringGrid.Cells[cbErrBarYlow.ItemIndex,j]    := StringGrid.Cells[cbErrBarYlow.ItemIndex,j];
    StringGrid.Cells[LastIndex_of_cbErrBarYlow,j] := StringGrid.Cells[LastIndex_of_cbErrBarYlow,j];
  end;
  LastIndex_of_cbErrBarYlow := cbErrBarYlow.ItemIndex;
end;

procedure TDataSeriesDialog.cbErrBarYhighChange(Sender: TObject);
var j: Integer;
begin
  if cbErrBarYhigh.ItemIndex > 0 then
    cbErrBarYhigh.Color := 16764663
  else
    cbErrBarYhigh.Color := clWhite;
  if (cbErrBarYlow.ItemIndex = 0) and (cbErrBarYhigh.ItemIndex > 0) then
  begin
    cbErrBarYlow.ItemIndex := cbErrBarYhigh.ItemIndex;
    cbErrBarYlowChange(Sender);
  end;
  for j := 1 to StringGrid.RowCount do
  begin
    StringGrid.Cells[cbErrBarYhigh.ItemIndex,j]    := StringGrid.Cells[cbErrBarYhigh.ItemIndex,j];
    StringGrid.Cells[LastIndex_of_cbErrBarYhigh,j] := StringGrid.Cells[LastIndex_of_cbErrBarYhigh,j];
  end;
  LastIndex_of_cbErrBarYhigh := cbErrBarYhigh.ItemIndex;
end;

procedure TDataSeriesDialog.StringGridExit(Sender: TObject);
begin
  StringGrid.DefaultDrawing := False;
end;

procedure TDataSeriesDialog.StringGridEnter(Sender: TObject);
begin
  StringGrid.DefaultDrawing := True;
end;

procedure TDataSeriesDialog.HelpButtonClick(Sender: TObject);
begin
  {$ifdef Linux}
  InvokeHelp;
  //Main.IndexClick(Sender);
  {$else}
  Keybd_Event(VK_F1, 0, 0, 0);
  {$endif}
end;

procedure TDataSeriesDialog.CancelButtonClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;


end.
