unit Global_Variables;

interface

uses
{$ifdef Linux}
  SysUtils, Variants, Classes, QGraphics, QControls, 
  QForms, QDialogs, QStdCtrls, QExtCtrls, QGrids, Qt, IniFiles, Grids,
  StdCtrls, Controls, ExtCtrls
  ;
{$else}
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Grids, IniFiles;
{$endif}

type
  TGlobalVariablesDialog = class(TForm)
    Panel1: TPanel;
    OK: TButton;
    CancelButton: TButton;
    HelpButton: TButton;
    Table: TStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure OKClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
  public
  end;

var
  GlobalVariablesDialog: TGlobalVariablesDialog;
  Msg: array[1..10] of String;

implementation

uses Grapher_Main;

{$R *.xfm}

procedure TGlobalVariablesDialog.FormCreate(Sender: TObject);
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
  i: Integer;
  procedure GetComboBoxItems(CB: TComboBox );
  begin
    i := CB.ItemIndex;
    CB.Items.Text := StringReplace(IniFile.ReadString(S, CB.Name +  '.Items.Text', ''), '<br>', #13#10 , [rfReplaceAll, rfIgnoreCase]);
    CB.ItemIndex := i;
  end;
begin
  Msg[1] := '   Name';
  Msg[2] := '   Value';
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  Font.Assign(Main.Font);
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        Caption                  := ReadString(S, 'Caption','');
        CancelButton.Caption     := ReadString(S, 'CancelButton.Caption','');
        HelpButton.Caption       := ReadString(S, 'HelpButton.Caption','');
        for i := 1 to 10 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>',   #13#10 , [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  Table.Cells[0,0] := Msg[1];
  Table.Cells[1,0] := Msg[2];
end;

procedure TGlobalVariablesDialog.OKClick(Sender: TObject);
var i: Integer;
begin
  for i := 1 to 6 do
  begin
    gvName[i]  := Trim(Table.Cells[0,i]);
    if gvName[i] <> '' then
      gvValue[i] := StrToFloat(Trim(Table.Cells[1,i]));
  end;
  Close;
end;

procedure TGlobalVariablesDialog.CancelButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TGlobalVariablesDialog.HelpButtonClick(Sender: TObject);
{$ifdef Linux}
//var S: String;
{$endif}
begin
  {$ifdef Linux}
  //S := '';
  //QKeyEvent_create(QEventType_KeyPress, Key_F1, 0, 0, @S, False, 0);
  //Main.IndexClick(Sender);
  InvokeHelp;
  {$else}
  Keybd_Event(VK_F1, 0, 0, 0);
  {$endif}
 // Application.HelpCommand(HELP_CONTEXT, 0);
end;

procedure TGlobalVariablesDialog.FormShow(Sender: TObject);
var i: Integer;
begin
  for i := 1 to 6 do
    if gvName[i] <> '' then
    begin
      Table.Cells[0,i] := gvName[i];
      Table.Cells[1,i] := FloatToStr(gvValue[i]);
    end;
end;

end.
