unit HelpTreeView;
{$ObjExportAll On}

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QComCtrls, IniFiles, Libc, QImgList, QClipbrd;//, LibcExec;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, IniFiles, ShellAPI;
{$endif}

type
  TBrowser = (Konqueror, Mozilla, Netscape);
  THelpTreeView = class(TTreeView)
  private
    FHelpDir: String;
    FBrowser: String;
    j, Mode: Integer;
    Slash, HelpDirectory: String;
    FN, KeyWord: array[0..300] of String;
    IL: TImageList;
    function GetTitle(FileName: String): String;
    procedure CallBrowser(URL: String);
    procedure LoadIcons;
    procedure SaveVisuallyDesignedContents;
  protected
    procedure DblClick; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ShowContents;
    procedure ShowIndex;
    procedure SearchIndex(S: String);
    procedure SearchText(S: String);
    function GetExeName: String;
  {$ifdef Linux}
    function AppHelp(HelpType: THelpType; HelpContext: THelpContext;
              const HelpKeyword, HelpFile: String; var Handled: Boolean): Boolean;
    function LibcSystem(Command: String): Integer;
  {$else}
    function AppHelp(Command: Word; Data: Longint; var CallHelp: Boolean): Boolean;
  {$endif}
  published
    property HelpDir: String read FHelpDir write FHelpDir;
    property Browser: String read FBrowser write FBrowser;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [THelpTreeView]);
end;

constructor THelpTreeView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  {$ifdef Linux}
  Slash := '/';
  Browser := 'Mozilla';
  {$else}
  Slash := '\';
  Self.ShowLines   := False;
  Self.ShowButtons := False;
  {$endif}
  HelpDir :=  Slash + 'Help';
  IL := TImageList.Create(Self);
  Mode := 1;
  ReadOnly := True;
end;

destructor THelpTreeView.Destroy;
begin
  IL.Free;
  inherited Destroy;
end;

{$ifdef Linux}
function THelpTreeView.AppHelp(HelpType: THelpType; HelpContext: THelpContext;
  const HelpKeyword, HelpFile: String; var Handled: Boolean): Boolean;
{$else}
function THelpTreeView.AppHelp(Command: Word; Data: Longint; var CallHelp: Boolean): Boolean;
{$endif}
var
  IniFile: TIniFile;
  i: Integer;
  S: String;
begin
  if Screen.ActiveForm <> nil then
  begin
    HelpDirectory := ExtractFileDir(GetExeName) + HelpDir + Slash;
    if (Screen.ActiveForm.ActiveControl <> nil) and (Screen.ActiveForm.ActiveControl.HelpContext > 0) then
    begin
      i := Screen.ActiveControl.HelpContext;
      {$ifdef Linux}
      if i = 0 then
        try
          i := StrToInt(Screen.ActiveControl.HelpKeyword);
        except
          i := 1;
        end;
      {$endif}
    end
    else
    begin
      i := Screen.ActiveForm.HelpContext;
      {$ifdef Linux}
      if i = 0 then
        try
          i := StrToInt(Screen.ActiveForm.HelpKeyword);
        except
          i := 1;
        end;
      {$endif}
    end;
    if i = 0 then
      i := 1;
    IniFile := TIniFile.Create(HelpDirectory + 'HelpContextID.txt');
    with IniFile do
    try
      S := ReadString('ID', IntToStr(i), 'introduction');
    finally
      IniFile.Free;
    end;
    CallBrowser(HelpDirectory + S + '.htm');
  end;
  Result := True;
end;

procedure THelpTreeView.ShowContents;
var
  i: Integer;
  S: String;
begin
  SaveVisuallyDesignedContents;
  HelpDirectory := ExtractFileDir(GetExeName) + HelpDir + Slash;
  S := HelpDirectory + 'Contents.txt';
  if IL.Count = 0 then
    LoadIcons;
  with Self do
    try
      {$ifdef Linux}
      if FileExists('/usr/bin/nkf') then
      begin
        LibcSystem('nkf -e ' + S + ' > ' + ChangeFileExt(S, ''));
        LoadFromFile(ChangeFileExt(S, ''));
        DeleteFile(ChangeFileExt(S, ''));
      end
      else
        LoadFromFile(S);
      {$else}
      LoadFromFile(S);
      {$endif}
      for i := 0 to Self.Items.Count - 1 do
      begin
        S := Items[i].Text;
        if Pos('=', S) > 0 then
        begin
          FN[i] := StringReplace(Copy(S, Pos('=', S) + 1, 100) + '.htm', ' ', '', [rfReplaceAll]);
          Self.Items[i].Text := Copy(S, 0, Pos('=', S) - 1);
        end
        else
          FN[i] := '';
      end;
      if (Self.Images = nil) and (IL.Count > 1) then
        Self.Images := IL;
      for i := 0 to Items.Count - 1 do
      begin
        if Items[i].HasChildren then
        begin
          Items[i].ImageIndex    := 0;
          Items[i].SelectedIndex := 0;
        end
        else
        begin
          Items[i].ImageIndex    := 2;
          Items[i].SelectedIndex := 2;
        end
      end;
    except
      ShowMessage('Sorry, failed to load contents file.');
    end;
end;

procedure THelpTreeView.SaveVisuallyDesignedContents;
var S: String;
begin
  HelpDirectory := ExtractFileDir(GetExeName) + HelpDir + Slash;
  S := HelpDirectory + 'Contents.txt';
  if (FileExists(S) = False) and (Self.Items.Count > 0) then
    Self.SaveToFile(S);
end;

procedure THelpTreeView.LoadIcons;
var  bmp: TBitmap;
  procedure ShowLinesAndRoot;
  begin
    if Self.Images = nil then
    begin
      {$ifndef Linux} // For Win
      Self.ShowLines := True;
      Self.ShowRoot  := True;
      Self.ShowButtons := True;
      {$endif}
    end;
  end;
begin
  bmp := TBitmap.Create;
  try
    bmp.TransparentMode := tmAuto;
    if FileExists(HelpDirectory + 'bookshut.bmp') then
    begin
      bmp.LoadFromFile(HelpDirectory + 'bookshut.bmp');
      IL.AddMasked(bmp, bmp.TransparentColor);
    end
    else
      ShowLinesAndRoot;
    if FileExists(HelpDirectory + 'bookopen.bmp') then
    begin
      bmp.LoadFromFile(HelpDirectory + 'bookopen.bmp');
      IL.AddMasked(bmp, bmp.TransparentColor);
    end
    else
      ShowLinesAndRoot;
    if FileExists(HelpDirectory + 'helpindex.bmp') then
    begin
      bmp.LoadFromFile(HelpDirectory + 'helpindex.bmp');
      IL.AddMasked(bmp, bmp.TransparentColor);
    end
    else
      ShowLinesAndRoot;
  finally
    bmp.Free;
  end;
end;

procedure THelpTreeView.ShowIndex;
var
  S: String;
  SL: TStringList;
  i: Integer;
begin
  SaveVisuallyDesignedContents;
  HelpDirectory := ExtractFileDir(GetExeName) + HelpDir + Slash;
  if IL.Count = 0 then
    LoadIcons;
  S := HelpDirectory + 'Index.txt';
  with Self do
  SL := TStringList.Create;
  try
    Self.Items.Clear;
    {$ifdef Linux}
    if FileExists('/usr/bin/nkf') then
    begin
      LibcSystem('nkf -e ' + S + ' > ' + ChangeFileExt(S, ''));
      SL.LoadFromFile(ChangeFileExt(S, ''));
      DeleteFile(ChangeFileExt(S, ''));
    end
    else
    {$endif}
    SL.LoadFromFile(S);
    SL.Sort;
    j := - 1;
    for i := 0 to SL.Count - 1 do
    begin
      S := SL[i];
      if Pos('=', S) > 0 then
      begin
        j := j + 1;
        FN[j] := StringReplace(Copy(S, Pos('=', S) + 1, 100) + '.htm', ' ', '', [rfReplaceAll]);
        Self.Items.Add(nil, Copy(S, 0, Pos('=', S) - 1));
      end;
    end;
  finally
    SL.Free;
  end;
  if (Self.Images = nil) and (IL.Count > 1) then
    Self.Images := IL;
  for i := 0 to Items.Count - 1 do
  begin
    Items[i].ImageIndex    := 2;
    Items[i].SelectedIndex := 2;
  end;
end;

function THelpTreeView.GetTitle(FileName: String): String;
var
  SL: TStringList;
  S: String;
  i: Integer;
begin
  FileName := ExtractFileDir(GetExeName) + FileName;
  j := j + 1;
  SL := TStringList.Create;
  try
    {$ifdef Linux}
    if FileExists('/usr/bin/nkf') then
    begin
      S := ChangeFileExt(FileName, '.txt');
      LibcSystem('nkf -e ' + FileName + ' > ' + S);
      SL.LoadFromFile(S);
      DeleteFile(S);
    end
    else
      SL.LoadFromFile(FileName);
    {$else}
    SL.LoadFromFile(FileName);
    {$endif}
    S := AnsiLowerCase(SL.Text);
    i := Pos('<title>', S);
    S:= Copy(SL.Text, i + 7, Pos('</title>', S) - i - 7);
    FN[j] := ExtractFileName(FileName);
    KeyWord[j] := S;
  finally
    SL.Free;
  end;
  Result := S;
end;

procedure THelpTreeView.DblClick;
var
  i: Integer;
  S: String;
begin
  i := Self.Selected.AbsoluteIndex;
  if Self.Items[i].HasChildren then
  begin
    if Self.Items[i].Expanded then
    begin
      Self.Items[i].ImageIndex    := 1;
      Self.Items[i].SelectedIndex := 1;
    end
    else
    begin
      Self.Items[i].ImageIndex    := 0;
      Self.Items[i].SelectedIndex := 0;
    end
  end
  else
  begin
    S := HelpDirectory + FN[Self.Selected.AbsoluteIndex];
    if FileExists(S) then
      CallBrowser(S);
  end;
  inherited DblClick;
end;

procedure THelpTreeView.SearchText(S: String);
var
  i: Integer;
  FileName: String;
  SR: TSearchRec;
  SL: TStringList;
  function GetFileStr(F: String): String;
  {$ifdef Linux}
  var S: String;
  {$endif}
  begin
    SL := TStringList.Create;
    try
      {$ifdef Linux}
      if FileExists('/usr/bin/nkf') then
      begin
        S := ChangeFileExt(F, '.txt');
        LibcSystem('nkf -e ' + F + ' > ' + S);
        SL.LoadFromFile(S);
        DeleteFile(S);
      end
      else
      {$endif}
      SL.LoadFromFile(F);
      Result := SL.Text;
    finally
      SL.Free;
    end;
  end;
begin
  SaveVisuallyDesignedContents;
  Self.Items.Clear;
  if IL.Count = 0 then
    LoadIcons;
  HelpDirectory := ExtractFileDir(GetExeName) + HelpDir + Slash;
  j := -1;
  if FindFirst(HelpDirectory + '*.htm', faAnyFile, SR) = 0 then
  begin
    FileName := HelpDirectory + SR.Name;
    if Pos(S, GetFileStr(FileName)) > 0 then
      Self.Items.Add(nil, GetTitle(HelpDir + Slash + SR.Name));
    while FindNext(SR) = 0 do
    begin
      FileName := HelpDirectory + SR.Name;
      if Pos(S, GetFileStr(FileName)) > 0 then
        Self.Items.Add(nil, GetTitle(HelpDir + Slash + SR.Name));
    end;
  end;
  FindClose(SR);
  if (Self.Images = nil) and (IL.Count > 1) then
    Self.Images := IL;
  for i := 0 to Items.Count - 1 do
  begin
    Items[i].ImageIndex    := 2;
    Items[i].SelectedIndex := 2;
  end
end;

procedure THelpTreeView.SearchIndex(S: String);
var i: Integer;
begin
  for i := 0 to Self.Items.Count - 1 do
  begin
    if Pos(S, Self.Items[i].Text) > 0 then
    begin
      Self.Items[i].Selected := True;
      Self.SetFocus;
    end;
  end;
end;

procedure THelpTreeView.CallBrowser(URL: String);
{$ifdef Linux}
var
  S: String;
{$endif}
begin
  {$ifdef Linux}
  S := GetEnvironmentVariable('HOME');
 // ShowMessage(Browser);
  if Browser = 'Konqueror' then
  begin
    if LibcSystem('kfmclient openURL "' + URL + '"') = -1 then
      LibcSystem('mozilla ' + URL + '&')
  end
  else
    if Browser = 'Mozilla' then
    begin
      if FileExists('/usr/bin/mozilla') then
        LibcSystem('mozilla ' + URL + '&')
      else
        LibcSystem('kfmclient openURL "' + URL + '"');
    end
    else
      if Browser = 'Netscape' then
      begin
        if FileExists('/usr/local/netscape/netscape') then
          LibcSystem('/usr/local/netscape/netscape ' + URL + '&')
        else
          LibcSystem('kfmclient openURL "' + URL + '"');
      end
      else
      begin
        if FileExists('/usr/bin/mozilla') then
          LibcSystem('mozilla ' + URL + '&')
        else
          if FileExists('/usr/local/netscape/netscape') then
            LibcSystem('/usr/local/netscape/netscape ' + URL + '&')
          else
            if LibcSystem('kfmclient openURL "' + URL + '"') = -1 then
   //         if (DirectoryExists(S + '/.kde')) and (DirectoryExists(S + '/Desktop')) then
              ShowMessage('Sorry, cannot open the browser.');
      end;
  {$else}
    ShellExecute(0, 'open', PChar(URL), nil, PChar(ExtractFileDir(URL)), 3);
  {$endif}
end;

{$ifdef Linux}
function THelpTreeView.LibcSystem(Command: String): Integer;
var
  Str: array [0..1000] of char;
begin
  StrPlCopy(Str, Command, 1000);
  Result := Libc.system(Str);
  //ShowMessage(Command + ' ' + IntToStr(Result));
end;
{$endif}

function THelpTreeView.GetExeName: String;
{$ifdef Linux}
var
  buffer: array [0..MAX_PATH] of Char;
  length: Integer;
{$endif}
begin
{$ifdef Linux}
  length := readlink('/proc/self/exe', buffer, MAX_PATH);
  if length > -1 then
    SetString(Result, buffer, length)
  else
    RaiseLastOSError;
{$else}
  Result := Application.ExeName;
{$endif}
end;
end.
