unit Help_Search;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, QComCtrls, HelpTreeView, IniFiles;
{$else}  
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, HelpTreeView, IniFiles;
{$endif}

type
  THelpSearch = class(TForm)
    TabControl: TTabControl;
    edSearch: TEdit;
    btnSearch: TButton;
    lblPrompt: TLabel;
    HelpTreeView: THelpTreeView;
    procedure FormShow(Sender: TObject);
    procedure TabControlChange(Sender: TObject);
    procedure btnSearchClick(Sender: TObject);
    procedure edSearchKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private

  public
  end;

var
  HelpSearch: THelpSearch;

implementation

{$R *.xfm}

procedure THelpSearch.FormShow(Sender: TObject);
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
  LANG: Integer;
{$ifdef Linux}
  function ConvertToCLXcharset(i: Integer): TFontCharset;
  begin
    Case i of
    0  : Result := fcsDefaultCharSet;
    128: Result := fcsSet_Ja;
    else Result := fcsDefaultCharSet;
    end;
  end;
{$endif}
begin
  LANG := 0;
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        LANG := ReadInteger(S, 'LANG', 0);
        {$ifdef Linux}
        TabControl.Tabs.Items[0].Caption := ReadString(S, 'TabControl.Tabs.Items0','');
        TabControl.Tabs.Items[1].Caption := ReadString(S, 'TabControl.Tabs.Items1','');
        TabControl.Tabs.Items[2].Caption := ReadString(S, 'TabControl.Tabs.Items2','');
        {$else}
        TabControl.Tabs[0] := ReadString(S, 'TabControl.Tabs.Items0','');
        TabControl.Tabs[1] := ReadString(S, 'TabControl.Tabs.Items1','');
        TabControl.Tabs[2] := ReadString(S, 'TabControl.Tabs.Items2','');
        {$endif}
        btnSearch.Caption := ReadString(S, 'btnSearch.Caption','');
        lblPrompt.Caption := ReadString(S, 'lblPrompt.Caption','');
        edSearch.Text     := ReadString(S, 'edSearch.Text','');
      finally
        Free;
      end;
  end;
{$ifdef Linux}
  Font.Charset := ConvertToCLXcharset(LANG);
{$else}
  Font.Charset := LANG;
{$endif}
  btnSearch.Enabled := False;
  TabControlChange(Sender);
end;

procedure THelpSearch.TabControlChange(Sender: TObject);
begin
  btnSearch.Enabled := False;
  case TabControl.TabIndex of
  0:begin
      HelpTreeView.ShowContents;
      {$ifdef Linux}
      HelpKeyword := '7';
      {$else}
      HelpContext := 7;
      {$endif}
      edSearch.Enabled := False;
    end;
  1:begin
      HelpTreeView.ShowIndex;
      {$ifdef Linux}
      HelpKeyword := '4';
      {$else}
      HelpContext := 4;
      {$endif}
      edSearch.Enabled := True;
    end;
  2:begin
      btnSearch.Enabled := True;
      {$ifdef Linux}
      HelpKeyword := '5';
      {$else}
      HelpContext := 5;
      {$endif}
      edSearch.Enabled := True;
      HelpTreeView.Items.Clear;
    end;
  end;
end;

procedure THelpSearch.btnSearchClick(Sender: TObject);
begin
  HelpTreeView.SearchText(edSearch.Text);
end;

procedure THelpSearch.edSearchKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  HelpTreeView.SearchIndex(edSearch.Text);
end;


end.
