unit HyperText;

interface

uses
{$ifdef Linux}
  Qt, SysUtils, Types, Classes, QForms, QControls, Variants, QGraphics, QDialogs, Math, QStdCtrls, QPrinters, IniFiles;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Printers, Math, IniFiles;
{$endif}

const ShadowColor = 10066329;

type
  THyperText = class(TComponent)
  private
    Msg: array[1..10] of String;
    FMagPrint, FMagScreen, FDisplaySize: Double;
    FDefaultFontSize: Shortint;
    FBGcolor: TColor;
    FAllowWrite: Boolean;
    FTargetCanvas: TCanvas;
    FFontNameA, FFontNameG, FFontNameM, FFontNameR, FFontNameS, FFontNameV: TFontName;
  protected

  public
    LocaleShift, Subscript, SymbolVshift: Shortint;
    x_end, y_end, LANG: Integer;
    Shadow: Boolean;
    SymbolWidthCorrector: Double;
    constructor Create(AOwner: TComponent); override;
    {$ifdef Linux}
    procedure Write(OriginalString: WideString; x, y, Angle: Integer);
    {$else}
    procedure Write(OriginalString: String; x, y, Angle: Integer);
    {$endif}
  published
    property MagPrint: Double read FMagPrint write FMagPrint;
    property MagScreen: Double read FMagScreen write FMagScreen;
    property DisplaySize: Double read FDisplaySize write FDisplaySize;
    property BGcolor: TColor read FBGcolor write FBGcolor;
    property DefaultFontSize: Shortint read FDefaultFontSize write FDefaultFontSize;
    property AllowWrite: Boolean read FAllowWrite write FAllowWrite;
    property TargetCanvas: TCanvas read FTargetCanvas write FTargetCanvas;
    property FontNameA: TFontName read FFontNameA write FFontNameA;
    property FontNameG: TFontName read FFontNameG write FFontNameG;
    property FontNameM: TFontName read FFontNameM write FFontNameM;
    property FontNameR: TFontName read FFontNameR write FFontNameR;
    property FontNameS: TFontName read FFontNameS write FFontNameS;
    property FontNameV: TFontName read FFontNameV write FFontNameV;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [THyperText]);
end;

constructor THyperText.Create(AOwner: TComponent);
var
  IniFile: TIniFile;
  ResourceFileName: String;
  i: Integer;
begin
  inherited Create(AOwner);
  MagPrint := 1;
  MagScreen := 1;
  DisplaySize := 14;
  DefaultFontSize := 9;
  TargetCanvas := Printer.Canvas;
  AllowWrite := True;
  BGcolor := clWhite;
  SymbolWidthCorrector := 0;
  SymbolVshift := 0;
  LANG := 0;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        LANG := ReadInteger('HyperText', 'LANG', 0);
        for i := 1 to 4 do
          Msg[i] := StringReplace(ReadString('HyperText', 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
{$ifdef Linux}
  FontNameA := 'nimbus sans l';//'cursor';
  FontNameR := 'utopia';
  FontNameS := 'standard symbols l';
  FontNameV := 'lucida';
{$else}
  FontNameA := 'Arial';
  FontNameR := 'Times New Roman';
  FontNameS := 'Symbol';
  FontNameV := 'Verdana';
{$endif}
end;

{$ifdef Linux}
procedure THyperText.Write(OriginalString: WideString; x, y, Angle: Integer);
{$else}
procedure THyperText.Write(OriginalString: String; x, y, Angle: Integer);
{$endif}
var
  IniFontSize, MaxFontSize, MaxHeight, i, j, k, xini, yini, qtshift: Integer;
  {$ifdef Linux}
    NewString, tag, NegativeTag, S: WideString;
  {$else}
    NewString, tag, NegativeTag, S: String;
    a0: Single;
    lf: TLogFont;
  {$endif}
  NonTag: Boolean;
  TempColor: TColor;
  mag, magqt, dum, Adjuster, Rad, dshift, dslide: Single;
  label JumpHere;
{$ifdef Linux}
  function ConvertToCLXcharset(i: Integer): TFontCharset;
  begin
    Case i of
    0  : Result := fcsDefaultCharSet;
    128: Result := fcsSet_Ja;
    else Result := fcsDefaultCharSet;
    end;
  end;
{$endif}
begin
  //asc := MagScreen * a0;
  if Copy(OriginalString, Length(OriginalString), 1) = ']' then
    OriginalString := OriginalString + ' ';
  with TargetCanvas do
  begin
    magqt := Sqrt(Screen.Height / 600) + 0.1;
    if TargetCanvas = Printer.Canvas then //Printing = True then
    begin
    {$ifdef Linux}
      Adjuster := 1;
      mag := MagPrint;
      magqt := 5;
      {
      QPainter_scale(Printer.Canvas.Handle, 5, 5);
      x := Trunc(x * 0.2);
      y := Trunc(y * 0.2);
      }
    {$else}
      a0 := Screen.Height / (0.6 * 25.4 * DisplaySize);
      Adjuster := 14.2 * GetDeviceCaps(Printer.Handle, LOGPIXELSX) / 360 / a0;
      mag := MagPrint;
    {$endif}
    end
    else
    begin
      Adjuster := 1;
      mag := MagScreen;//  Adjuster / a0;
    end;
    //--- Getting Maximum FontSize ---//
    i := Trunc(DefaultFontSize * magqt);
    //Shadow := False;
    S := OriginalString;
    if Pos('[1]', S) > 0 then i := Trunc( 8 * magqt);
    if Pos('[2]', S) > 0 then i := Trunc( 9 * magqt);
    if Pos('[3]', S) > 0 then i := Trunc(10 * magqt);
    if Pos('[4]', S) > 0 then i := Trunc(11 * magqt);
    if Pos('[5]', S) > 0 then i := Trunc(12 * magqt);
    if Pos('[6]', S) > 0 then i := Trunc(14 * magqt);
    if Pos('[7]', S) > 0 then i := Trunc(16 * magqt);
    if Pos('[8]', S) > 0 then i := Trunc(20 * magqt);
    if Pos('[9]', S) > 0 then i := Trunc(24 * magqt);
    MaxFontSize := i;
    //--------------------------------//
    Brush.Style := bsClear;
    i := Length(S);
    if   (ByteType(S, i div 4) <> mbSingleByte)
      or (ByteType(S, i div 3) <> mbSingleByte)
      or (ByteType(S, i div 2) <> mbSingleByte)
      or (ByteType(S, i)       <> mbSingleByte)
      then
    begin
      S := Font.Name;
      if (S <> FontNameG) and (S <> FontNameM) then
      begin
        Font.Name := FontNameG;
        {$ifdef Linux}
        Font.Charset := ConvertToCLXcharset(LANG);
        {$else}
        Font.Charset := LANG;
        {$endif}
      end;
    end;
   // Font.Size := IniFontSize;
   // Font.Name := 'Times New Roman';
    dum := Font.Size;
    IniFontSize := Ceil(dum / mag); // 2/9
    //IniFontSize := Trunc(dum * mag); // 2/9
    {
    Font.Name := NumFontName;
    Font.Size := Ceil(DefaultFontSize * mag);
    Font.Charset := 0;
    Font.Style := [];
    }
    Subscript := 0;
    LocaleShift := 0;
    {$ifdef Linux}
    j := Pos('[', OriginalString);
    {$else}
    j := AnsiPos('[', OriginalString);
    {$endif}
    //--------------00/01/29
    if Copy(OriginalString, j - 1, 1) <>  '@' then
    begin
      NonTag := False;
      S := Copy(OriginalString, 1, j - 1);
      if S = '' then
        NewString := OriginalString
      else
        NewString := Copy(OriginalString, j + 3, Length(OriginalString));
    end
    else
    begin
      NonTag := True;
      S := Copy(OriginalString, j+1, 500);
      {$ifdef Linux}
      k := Pos('[', S) - 1;
      {$else}
      k := AnsiPos('[', S) - 1;
      {$endif}
      if k = -1 then k := 0;
      S := Copy(OriginalString, 1, j-2) + '[' + Copy(S, 1, k);
      NewString := Copy(OriginalString, j + k + 1, Length(OriginalString));
    end;
    //--------------
    dshift := 0;
    dslide := 0;
    Subscript := 0;
  {$ifdef Linux}
    TargetCanvas.Start;//(False);
    xini := x;
    yini := y;
    x := 0;
    y := 0;
    QPainter_translate(Handle, xini, yini);
    QPainter_rotate(Handle, - Angle);
    Rad := 0;
  {$else}
    xini := 0;
    yini := 0;
    Angle := Angle * 10;
    Rad   := Angle / 1800 * pi;
    FillChar(lf, SizeOf(lf), 0);   //
    with lf do                       //
    begin                            //
      lfHeight := Font.Height;//
      lfEscapement := Angle;                 //
      lfWeight := 700 * Ord(fsBold in Font.Style);  //
      lfItalic := Ord(fsItalic in Font.Style);  //
      lfCharSet := Font.Charset;
      StrPCopy(lfFaceName, Font.Name);          //
      Font.Handle := CreateFontIndirect(lf);    //
    end;                                                 //
    if Font.Charset = LANG then
    begin
      LocaleShift := Ceil(1 * mag * Adjuster * magqt);
      dshift := dshift + LocaleShift; // IniFontSize div 5;
    end;
  {$endif}
   // dshift := dshift + Trunc((MaxDefaultFontSize - IniFontSize) * mag);     // 7/2
    dshift := dshift + (MaxFontSize - IniFontSize) * mag * Adjuster;
    if (Pen.Color <> clWhite) or (Pen.Color <> clNavy) then
      if AllowWrite = True then //!
      //=================================================================================//
        TextOut(x + Trunc(dshift * sin(Rad)), y + Trunc(dshift * cos(Rad)), S);
      //=================================================================================//
 //   QPainter_rotate(TargetCanvas.Handle, Angle / 10);
    x := x + Trunc(TextWidth(S) * cos(Rad));                               //
    y := y - Trunc(TextWidth(S) * sin(Rad));                               //
    MaxHeight := TextHeight(S);
    repeat
      if (j > 0) and (NonTag = False) then
      begin
        tag := Copy(OriginalString, j + 1, 1);
        if tag = '/' then
        begin
          NegativeTag := Copy(OriginalString, j + 2, 1);
          NewString := Copy(NewString, 2, Length(NewString));
        end;
      end
      else
        tag := '';
      with Font do
      begin
        if tag = 'x' then
        begin
          NewString := '';
          goto JumpHere;
        end;
        if tag <> '/' then
        begin
          if tag = '0' then IniFontSize := Trunc( 7 * magqt);
          if tag = '1' then IniFontSize := Trunc( 8 * magqt);
          if tag = '2' then IniFontSize := Trunc( 9 * magqt);
          if tag = '3' then IniFontSize := Trunc(10 * magqt);
          if tag = '4' then IniFontSize := Trunc(11 * magqt);
          if tag = '5' then IniFontSize := Trunc(12 * magqt);
          if tag = '6' then IniFontSize := Trunc(14 * magqt);
          if tag = '7' then IniFontSize := Trunc(16 * magqt);
          if tag = '8' then IniFontSize := Trunc(20 * magqt);
          if tag = '9' then IniFontSize := Trunc(24 * magqt);
          if tag = 'K' then
          begin
            if BGcolor <> clWhite then
              Color := clWhite
            else
              Color := clBlack;
          end;
          if tag = 'A' then Color := clAqua;
          if tag = 'B' then Color := clBlue;
          if tag = 'D' then Color := clDkGray;
          if tag = 'F' then Color := clFuchsia;
          if tag = 'G' then Color := clGreen;
          if tag = 'L' then Color := clLime;
          if tag = 'M' then Color := clMaroon;
          if tag = 'N' then Color := clNavy;
          if tag = 'O' then Color := clOlive;
          if tag = 'P' then Color := clPurple;
          if tag = 'R' then Color := clRed;
          if tag = 'S' then Color := clSilver;
          if tag = 'T' then Color := clTeal;
          if tag = 'Y' then Color := clYellow;
        {$ifdef Linux}
          if tag = 'a' then begin; Name := FontNameA ; Charset := fcsLatin1; end; //
          if tag = 'r' then begin; Name := FontNameR ; Charset := fcsLatin1; end; //
          if tag = 's' then begin; Name := FontNameS ; Charset := fcsAnyCharset; end; //
          if tag = 'v' then begin; Name := FontNameV ; Charset := fcsLatin1; end; //
          if LANG = 128 then
          begin
            if tag = 'g' then begin; Name := FontNameG  ; Charset := fcsSet_Ja; end; //
            if tag = 'm' then begin; Name := FontNameM  ; Charset := fcsSet_Ja; end; //
          end;
        {$else}
          if tag = 'r' then begin; Name := 'Times New Roman'; Charset :=   0; end; //
          if tag = 's' then begin; Name := 'Symbol'         ; Charset :=   0; end; //
          if tag = 'v' then begin; Name := 'Verdana'        ; Charset :=   0; end; //
          if tag = 'a' then begin; Name := 'Arial'          ; Charset :=   0; end; //
          if LANG = 128 then
          begin
            if tag = 'g' then begin; Name := FontNameG; Charset := SHIFTJIS_CHARSET; end; //
            if tag = 'm' then begin; Name := FontNameM; Charset := SHIFTJIS_CHARSET; end; //
          end;
        {$endif}
          if tag = 'i' then
            Style := Style + [fsItalic];
          if tag = 'b' then
            Style := Style + [fsBold];
          if tag = ' ' then
            Style := [];
          if tag = '_' then
            if Subscript <> 0 then
              dslide := dslide + 1
            else
              Subscript :=  1;
          if tag = '^' then
            if Subscript <> 0 then
              dslide := dslide - 1
            else
              Subscript := -1;
          if tag = '-' then
          begin;
            Subscript :=  0;
            dslide := 0;
          end;
          if tag = '~' then dslide := dslide - Trunc(2 * mag * Adjuster);
          if tag = '.' then dslide := dslide + Trunc(2 * mag * Adjuster);
          Case Subscript of
           1: Size := Trunc(IniFontSize * 0.7);
           0: Size := IniFontSize;
          -1: Size := Trunc(IniFontSize * 0.7);
          end;
          Size := Trunc(Size * mag);
          if tag = '#' then Shadow := True;
          if tag = ' ' then Shadow := False;
        end
        else
        begin
          if NegativeTag = 'b' then Style := Style - [fsBold];
          if NegativeTag = 'i' then Style := Style - [fsItalic];
          if NegativeTag = '#' then Shadow := False;
        end;
        Case Subscript of
         1: dshift := Trunc(Size * 0.72 * Adjuster);
         0: dshift := 0;
        -1: dshift := 0;// Ceil(Size * 0.05);
        end;
        {$ifdef Linux}
        if Name = 'symbol' then  //) and (TargetCanvas = Printer.Canvas)
          dshift := dshift + Ceil(SymbolVshift * magqt);
        //if Name = 'times' then
        //  dshift := dshift + Ceil(2 * magqt);
        {$else}
        if Name = 'Symbol' then
          dshift := dshift - Ceil(Size * 0.1);// * Adjuster / asc)
        if Charset = SHIFTJIS_CHARSET then
          dshift := dshift - Ceil(Size * 0.2);
        {$endif}
       // dshift := dshift + Trunc((DefaultFontSize - IniFontSize) * mag);     // 7/2
        dshift := dshift + Trunc((MaxFontSize - IniFontSize) * mag * Adjuster);
      end;
      dshift := dshift + dslide;
    {$ifdef Linux}
      // ???
      {
      QPainter_translate(Handle, xini, yini);
      QPainter_rotate(Handle, - Angle);
      Rad := 0;
      }
    {$else}
      if Font.Charset = LANG then
      begin
        LocaleShift := Ceil(1 * magqt);
        dshift := dshift + LocaleShift; // IniFontSize div 5;
      end;
      FillChar(lf, SizeOf(lf), 0);   //
      with lf do                       //
      begin                            //
        lfHeight := Font.Height;//
        lfEscapement := Angle;                 //
        lfWeight := 700 * Ord(fsBold in Font.Style);  //
        lfItalic := Ord(fsItalic in Font.Style);  //
        lfCharSet := Font.Charset;
        StrPCopy(lfFaceName, Font.Name);          //
        Font.Handle := CreateFontIndirect(lf);    //
      end;                                                 //
    {$endif}
    {$ifdef Linux}
      j := Pos('[', NewString);
    {$else}
      j := AnsiPos('[', NewString);
    {$endif}
      //----------- 00/01/29
      if j = 0  then break;
      if Copy(NewString, j - 1, 1) <>  '@' then
      begin
        NonTag := False;
        S := Copy(NewString, 1, j-1);
      end
      else
      begin
        NonTag := True;
        S := Copy(NewString, j, 500);
      {$ifdef Linux}
        k := Pos('[', S) - 1;
      {$else}
        k := AnsiPos('[', S) - 1;
      {$endif}
        S := Copy(NewString, 1, j-2)+ '[' + Copy(S, 1, k);
        j := j - 2;
      end;
      //-----------
      if (Pen.Color <> clWhite) or (Pen.Color <> clNavy) then
        if AllowWrite = True then //!
        begin
          if Shadow then
          begin
            TempColor  := Font.Color;
            Font.Color := ShadowColor;
            i := Trunc(Adjuster * (Font.Size div 6));
          //=========================================//
            TextOut(x + i, y + i + Trunc(dshift), S);
          //=========================================//
            Font.Color := TempColor;
            (*
            {$ifdef Linux}
            if Font.Charset = ConvertToCLXcharset(LANG) then
            {$else}
            if Font.Charset = LANG then
            {$endif}
              y := y - Trunc(cos(Rad) * i) - 1; // Not knowing the reason, but nessesary.
            *)
          end;
        //=================================================================================//
          TextOut(x + Trunc(dshift * sin(Rad)), y + Trunc(dshift * cos(Rad)), S);
        //=================================================================================//
        end;
        i := TextWidth(S);
        {$ifdef Linux}
        if (Font.Name = 'symbol') and (TargetCanvas = Printer.Canvas) then
          i := i + Ceil(i * SymbolWidthCorrector);
        {$endif}
        x := x + Trunc(i * cos(Rad));                               //
        y := y - Trunc(i * sin(Rad));                               //
        i := TextHeight(S);
        if MaxHeight < i then
          MaxHeight := i;
      //  ShowMessage(IntToStr(TextHeight('c')) + ' ' + IntToStr(Font.Size));
      OriginalString := NewString;
      NewString := Copy(OriginalString, j + 3, Length(OriginalString))
    until Length(NewString) = 0;

    if (Pen.Color <> clWhite) or (Pen.Color <> clNavy) then
      if AllowWrite = True then //!
      begin
        if Shadow then
        begin
          TempColor  := Font.Color;
          Font.Color := ShadowColor;
          i := Trunc(Adjuster * (Font.Size div 6));
        //=================================================//
          TextOut(x + i, y + i + Trunc(dshift), NewString);
        //=================================================//
          Font.Color := TempColor;
        end;
      //==============================================================================//
        TextOut(x + Trunc(dshift * sin(Rad)), y + Trunc(dshift * cos(Rad)), NewString);
      //========================================================================//
      end;
      i := TextHeight(NewString);
      if MaxHeight < i then
        MaxHeight := i;
  JumpHere:
      x_end := x + Trunc(TextWidth(NewString) * cos(Rad)) + Trunc(MaxHeight * sin(Rad));                               //
      y_end := y - Trunc(TextWidth(NewString) * sin(Rad)) + Trunc(MaxHeight * cos(Rad));                               //
    //y := y + Ceil(Font.Size * 2);
  {$ifdef Linux}
    QPainter_rotate(Handle, Angle);
    QPainter_translate(Handle, -xini, -yini);
    TargetCanvas.Stop;
    x := x_end;
    x_end := Trunc(  x * cos(Angle / 180 * pi)) + Trunc(MaxHeight * sin(Angle / 180 * pi)) + xini;
    y_end := Trunc(- x * sin(Angle / 180 * pi)) + Trunc(MaxHeight * cos(Angle / 180 * pi)) + yini;
    if TargetCanvas = Printer.Canvas then
    begin
      {
      QPainter_scale(Printer.Canvas.Handle, 72/360, 72/360);
      x_end := Trunc(x_end * 5);
      y_end := Trunc(y_end * 5);
      }
    end;
  {$endif}
  end;                                                         //
end;


end.
