{

     TMathsParser version 1.32
     for parsing and evaluating simple mathematical expressions specified at runtime
     Copyright(C) 2000-2001 Haruyuki Fujimaki
     fujimaki@sakura.cc.tsukuba.ac.jp
     TQG00473@nifty.com
     URL: http://homepage2.nifty.com/Fujimaki/

}
{$ObjExportAll On}
unit MathsParser;

interface

uses
{$ifdef Linux}
  SysUtils, Types, Classes, QForms, QControls, Variants, QGraphics, QDialogs, Math, QStdCtrls, QPrinters, IniFiles;
{$else}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Math, IniFiles;
{$endif}

type
  EMissMatchingBracket = class(Exception);
  ESyntaxError = class(Exception);
  EInvalidInterval = class(Exception);
  TMathsParser = class(TComponent)
  private
    P: array[0..199] of Double;
    Operation, Former, Latter, Sign: array[0..199] of Integer;
    Msg: array[1..10] of String;
    NumOperation, NumParam, k: Integer;
    FExpression: String;
    FV1, FV2, FV3, FV4, FV5, FV6, FV7, FV8: Double;
    FV1Name, FV2Name, FV3Name, FV4Name, FV5Name, FV6Name, FV7Name, FV8Name: String;
    function PowerFunction(x: String): String;
    function Multiplication(x: String): String;
    function Division(x: String): String;
    function Subtraction(x: String): String;
    function Addition(x: String): String;
    function GetReducedExpression(i: Integer; x: String): String;
    function GetDoubleValue: Double;
    function CountString(Substr, S: String): Integer;
    function Erfc(x: Double): Double;
    function GetValueName(S: String): String;
    function f(x: Double): Double;
    procedure SetExpression(FExpression: String);
  public
    ErrMsg: String;
    Ready: Boolean;
    RankingOfLength: array[1..8] of Integer;
    NumCalc: Integer;
    constructor Create(AOwner: TComponent); override;
   // destructor Destroy(AOwner: TComponent); override;
    procedure ShowHelp;
    procedure DetectVariables(Expression: String; Exception: String = '');
    procedure SortLengthOfVariable;
    function FindRoot(Expression: String; LowerBoundary, UpperBoundary, Accuracy: Double): Double;
    function FindOptimumValue(Expression: String; LowerBoundary, UpperBoundary, Accuracy: Double; sign: Integer): Double;
    function Integrate(Expression: String; LowerBoundary, UpperBoundary: Double; Division: Integer): Double;
  published
    property V1: Double read FV1 write FV1;
    property V2: Double read FV2 write FV2;
    property V3: Double read FV3 write FV3;
    property V4: Double read FV4 write FV4;
    property V5: Double read FV5 write FV5;
    property V6: Double read FV6 write FV6;
    property V7: Double read FV7 write FV7;
    property V8: Double read FV8 write FV8;
    property V1Name: String read FV1Name write FV1Name;
    property V2Name: String read FV2Name write FV2Name;
    property V3Name: String read FV3Name write FV3Name;
    property V4Name: String read FV4Name write FV4Name;
    property V5Name: String read FV5Name write FV5Name;
    property V6Name: String read FV6Name write FV6Name;
    property V7Name: String read FV7Name write FV7Name;
    property V8Name: String read FV8Name write FV8Name;
    property Expression: String read FExpression write SetExpression;
    property Value: Double read GetDoubleValue;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TMathsParser]);
end;

constructor TMathsParser.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Msg[1] := 'Number of bracket is miss-matching. Review your expression.';
  Msg[2] := 'The expression is improper. Please check it.';
  Msg[3] := 'You specified invalid initial range. No root or plural root lies in the range.';
  Msg[4] :=
    'Three types of bracketing, ( ), [ ], { }, can be used.' + #13#10 +
    'Both the order and number of use are free. ' + #13#10 +
    'You can use blanks.' + #13#10 +
    'Accepted operators are + , - , * , /  and ^.' + #13#10 +
    'It does not matter if you use lower or upper case in typing variables and functions.' + #13#10 +
    '' + #13#10 +
    '==== Operator Precedence ====' + #13#10 +
    'If you combine several operators in an expression, the operations are performed in the order: ' + #13#10 +
    'Operator  Description' + #13#10 +
    '    -	    Negation (as in -1)' + #13#10 +
    '    ^         Exponentiation' + #13#10 +
    '* and /     Multiplication and division' + #13#10 +
    '+ and -     Addition and subtraction' + #13#10 +
    'To change the order of evaluation, enclose the part of the formula to be calculated first in parentheses.' + #13#10 +
    '' + #13#10 +
    '==== Supported Functions =====' + #13#10 +
    'abs( )      the absolute value of a number' + #13#10 +
    'arccos( )  the arccosine of a number' + #13#10 +
    'arcsin( )   the arcsine of a number' + #13#10 +
    'arctan( )   the arctangent of a number' + #13#10 +
    'cos( )      the cosine of a number' + #13#10 +
    'erf( )       error function' + #13#10 +
    'erfc( )     complementary error function' + #13#10 +
    'exp( )      the exponential of a number' + #13#10 +
    'ln( )        the natural logarithm of a number' + #13#10 +
    'log10( )    the base-10 logarithm of a number' + #13#10 +
    'sqrt( )      a positive square root of a number' + #13#10 +
    'sin( )       the sine of a number' + #13#10 +
    'tan( )       the tangent of a number' + #13#10 +
    '> The angle in trigonometric functions must be given in radians.' + #13#10 +
    '' + #13#10 +
    '==== Predefined Constant ====' + #13#10 +
    'pi            3.14159265..';
  V1Name := 'a';
  V2Name := 'b';
  V3Name := 'c';
  V4Name := 'd';
  V5Name := 'e';
  V6Name := 'f';
  V7Name := 'g';
  V8Name := 'x';
  Ready := False;
end;

procedure TMathsParser.SetExpression(FExpression: String);
var
  IniFile: TIniFile;
  ResourceFileName: String;
  OldEx, dumstr, oldstr, CalledFunction, PostStr: String;
  dumint, i, j, l: Integer;
  str, LocalValue : String;
  ParamName: array [1..8] of String;
begin
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        for i := 1 to 4 do
          Msg[i] := StringReplace(ReadString('MathsParser', 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  //--- Eliminating repeated parameter name ---//
  ParamName[1] := V1Name;
  ParamName[2] := V2Name;
  ParamName[3] := V3Name;
  ParamName[4] := V4Name;
  ParamName[5] := V5Name;
  ParamName[6] := V6Name;
  ParamName[7] := V7Name;
  ParamName[8] := V8Name;
  for i := 1 to 7 do
    for j := i + 1 to 8 do
      if ParamName[j] = ParamName[i] then
        ParamName[j] := '';
  V1Name := ParamName[1];
  V2Name := ParamName[2];
  V3Name := ParamName[3];
  V4Name := ParamName[4];
  V5Name := ParamName[5];
  V6Name := ParamName[6];
  V7Name := ParamName[7];
  V8Name := ParamName[8];
  //-------------------------------------------//
  Ready := False;
  str := '(';
  j := 10;
  FExpression := AnsiLowerCase(FExpression);
  FExpression := '(' + StringReplace(FExpression, ' ', '', [rfReplaceAll]) + ')';
  FExpression := StringReplace(FExpression, '[', '(', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, ']', ')', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, '{', '(', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, '}', ')', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, ')(', ')*(', [rfReplaceAll]);
  if CountString('(', FExpression) <> CountString(')', FExpression) then
  begin
    ErrMsg := Msg[1];
    raise EMissMatchingBracket.Create(ErrMsg);
    Exit;
  end;
  FExpression := StringReplace(FExpression, '+-', '-', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, '(-'   , '(0-', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'log10',   '@@@', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'sqrt' ,   '@@#', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'ln'   ,   '@@$', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'exp'  ,   '@#@', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'arcsin' , '@##', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'arccos' , '@#$', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'arctan' , '@$@', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'sin'  ,   '@$#', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'cos'  ,   '@$$', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'tan'  ,   '#@@', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'abs'  ,   '#@#', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'erfc' ,   '#@$', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'erf'  ,   '##@', [rfReplaceAll]);
  FExpression := StringReplace(FExpression, 'pi'  , FloatToStr(pi), [rfReplaceAll]);
  SortLengthOfVariable;
  for i := 1 to 8 do
  begin
    if (V1Name <> '') and (RankingOfLength[1] = i) then
      FExpression := StringReplace(FExpression, AnsiLowerCase(V1Name)  , '!!', [rfReplaceAll]);
    if (V2Name <> '') and (RankingOfLength[2] = i) then
      FExpression := StringReplace(FExpression, AnsiLowerCase(V2Name)  , '!&', [rfReplaceAll]);
    if (V3Name <> '') and (RankingOfLength[3] = i) then
      FExpression := StringReplace(FExpression, AnsiLowerCase(V3Name)  , '!%', [rfReplaceAll]);
    if (V4Name <> '') and (RankingOfLength[4] = i) then
      FExpression := StringReplace(FExpression, AnsiLowerCase(V4Name)  , '&!', [rfReplaceAll]);
    if (V5Name <> '') and (RankingOfLength[5] = i) then
      FExpression := StringReplace(FExpression, AnsiLowerCase(V5Name)  , '&&', [rfReplaceAll]);
    if (V6Name <> '') and (RankingOfLength[6] = i) then
      FExpression := StringReplace(FExpression, AnsiLowerCase(V6Name)  , '&%', [rfReplaceAll]);
    if (V7Name <> '') and (RankingOfLength[7] = i) then
      FExpression := StringReplace(FExpression, AnsiLowerCase(V7Name)  , '%!', [rfReplaceAll]);
    if (V8Name <> '') and (RankingOfLength[8] = i) then
      FExpression := StringReplace(FExpression, AnsiLowerCase(V8Name)  , '%&', [rfReplaceAll]);
  end;
  for i := 2 to Length(FExpression) do
  begin
  if (Ord(FExpression[i]) >= 48) and (Ord(FExpression[i]) <= 57) then
    LocalValue := LocalValue + FExpression[i]
  else
    if Ord(FExpression[i]) = 46 then
      LocalValue := LocalValue + FExpression[i]
    else
    begin
      if LocalValue <> '' then
      begin
        P[j] := StrToFloat(LocalValue);
        str := str + FormatFloat('00', j) + FExpression[i];
        j := j + 1;
      end
      else
        str := str + FExpression[i];
      LocalValue := '';
    end;
  end;
  if V1Name <> '' then
    str := StringReplace(str, '!!' , '01', [rfReplaceAll]);
  if V2Name <> '' then
    str := StringReplace(str, '!&' , '02', [rfReplaceAll]);
  if V3Name <> '' then
    str := StringReplace(str, '!%' , '03', [rfReplaceAll]);
  if V4Name <> '' then
    str := StringReplace(str, '&!' , '04', [rfReplaceAll]);
  if V5Name <> '' then
    str := StringReplace(str, '&&' , '05', [rfReplaceAll]);
  if V6Name <> '' then
    str := StringReplace(str, '&%' , '06', [rfReplaceAll]);
  if V7Name <> '' then
    str := StringReplace(str, '%!' , '07', [rfReplaceAll]);
  if V8Name <> '' then
    str := StringReplace(str, '%&' , '08', [rfReplaceAll]);
  //---------  Syntax Check ------------------------------------
  dumstr := str;
  dumstr := StringReplace(dumstr, '@@@', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '@@#', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '@@$', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '@#@', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '@##', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '@#$', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '@$@', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '@$#', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '@$$', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '#@@', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '#@#', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '#@$', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '##@', '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '' ,   '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '('  , '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, ')'  , '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '^'  , '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '*'  , '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '/'  , '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '+'  , '', [rfReplaceAll]);
  dumstr := StringReplace(dumstr, '-'  , '', [rfReplaceAll]);
  for i := 1 to Length(dumstr) do
  begin
    if (Ord(dumstr[i]) < 48) or (Ord(dumstr[i]) > 57) then
      if Ord(dumstr[i]) <> 46 then
      begin
        ErrMsg := Msg[2];
        raise ESyntaxError.Create(ErrMsg);
        Exit;
      end;
  end;
  //-----------------------------------------------------
  NumParam := j;
  k := 0;
  FExpression := str;
  repeat
    OldEx := FExpression;
    dumstr := FExpression;
    i := 0;
    repeat
      dumint := Pos('(' , dumstr);
      dumstr := Copy(dumstr, dumint + 1, Length(dumstr));
      i := i + dumint;
    until dumint = 0;
    PostStr := Copy(dumstr, Pos(')' , dumstr) + 1, Length(dumstr));
    if dumstr = PostStr then Break;
    dumstr := Copy(dumstr, 1, Pos(')' , dumstr) - 1);
    repeat
      oldstr := dumstr;
      dumstr := PowerFunction(oldstr);
    until dumstr = oldstr;
    repeat
      oldstr := dumstr;
      dumstr := Division(oldstr);
    until dumstr = oldstr;
    repeat
      oldstr := dumstr;
      dumstr := Multiplication(oldstr);
    until dumstr = oldstr;
    repeat
      //dumstr := StringReplace(dumstr, '+-', '-', [rfReplaceAll]);
      oldstr := dumstr;
      dumstr := Subtraction(oldstr);
    until dumstr = oldstr;
    repeat
      oldstr := dumstr;
      dumstr := Addition(oldstr);
    until dumstr = oldstr;
    CalledFunction := Copy(FExpression, i - 3, 3) ;
    l := StrToInt(dumstr);
    j := 4;
    k := k + 1;
    Former[k] := l;
    if      CalledFunction = '@#@' then Operation[k] :=  6    // Exp
    else if CalledFunction = '@@$' then Operation[k] :=  7    // ln
    else if CalledFunction = '@@@' then Operation[k] :=  8    // log10
    else if CalledFunction = '@$#' then Operation[k] :=  9    // sin
    else if CalledFunction = '@$$' then Operation[k] := 10    // cos
    else if CalledFunction = '#@@' then Operation[k] := 11    // tan
    else if CalledFunction = '#@#' then Operation[k] := 12    // Abs
    else if CalledFunction = '@##' then Operation[k] := 13    // ArcSin
    else if CalledFunction = '@#$' then Operation[k] := 14    // ArcCos
    else if CalledFunction = '@$@' then Operation[k] := 15    // ArcTan
    else if CalledFunction = '##@' then Operation[k] := 16    // 1 - Erfc
    else if CalledFunction = '#@$' then Operation[k] := 17    // Erfc
    else if CalledFunction = '@@#' then Operation[k] := 18    // Sqrt
    else
    begin
      k := k - 1;
      j := 1;
    end;
    FExpression := Copy(FExpression, 1, i - j) + FormatFloat('00', k + NumParam) + PostStr;
  until Pos('(', FExpression) = 0;
  dumstr := FExpression;
  repeat
    oldstr := dumstr;
    dumstr := PowerFunction(oldstr);
  until dumstr = oldstr;
  repeat
    oldstr := dumstr;
    dumstr := Division(oldstr);
  until dumstr = oldstr;
  repeat
    oldstr := dumstr;
    dumstr := Multiplication(oldstr);
  until dumstr = oldstr;
  repeat
  //  dumstr := StringReplace(dumstr, '+-', '-', [rfReplaceAll]);
    oldstr := dumstr;
    dumstr := Subtraction(oldstr);
  until dumstr = oldstr;
  repeat
    oldstr := dumstr;
    dumstr := Addition(oldstr);
  until dumstr = oldstr;
  NumOperation := k;
  Ready := True;
end;

procedure TMathsParser.SortLengthOfVariable;
var
  i: Integer;
  SL: TStringList;
begin
  SL := TStringList.Create;
  try
    SL.Add(IntToStr(Length(V1Name))+'1');
    SL.Add(IntToStr(Length(V2Name))+'2');
    SL.Add(IntToStr(Length(V3Name))+'3');
    SL.Add(IntToStr(Length(V4Name))+'4');
    SL.Add(IntToStr(Length(V5Name))+'5');
    SL.Add(IntToStr(Length(V6Name))+'6');
    SL.Add(IntToStr(Length(V7Name))+'7');
    SL.Add(IntToStr(Length(V8Name))+'8');
    SL.Sort;
    for i := 0 to 7 do
      RankingOfLength[StrToInt(Copy(SL.Strings[i], 2, 1))] := 8 - i;
   // ShowMessage(IntToStr(RankingOfLength[1]));
  finally
    SL.Free;
  end;
end;

function TMathsParser.PowerFunction(x: String): String;
var
  i : Integer;
begin
  i := Pos('^', x);
  if i > 0 then
  begin
    k := k + 1;
    Operation[k] := 1;
    Result := GetReducedExpression(i, x);
  end;
end;

function TMathsParser.Multiplication(x: String): String;
var
  i : Integer;
begin
  i := Pos('*', x);
  if i > 0 then
  begin
    k := k + 1;
    Operation[k] := 2;
    Result := GetReducedExpression(i, x);
  end;
end;

function TMathsParser.Division(x: String): String;
var
  i : Integer;
begin
  i := Pos('/', x);
  if i > 0 then
  begin
    k := k + 1;
    Operation[k] := 3;
    Result := GetReducedExpression(i, x);
  end;
end;

function TMathsParser.Addition(x: String): String;
var
  i: Integer;
begin
  i := Pos('+', x);
  if i > 0 then
  begin
    k := k + 1;
    Operation[k] := 4;
    Result := GetReducedExpression(i, x);
  end;
end;

function TMathsParser.Subtraction(x: String): String;
var
  i : Integer;
begin
  i := Pos('-', x);
  if i > 0 then
  begin
    k := k + 1;
    Operation[k] := 5;
    Result := GetReducedExpression(i, x);
  end;
end;

function TMathsParser.GetReducedExpression(i: Integer; x: String): String;
begin
  Former[k]   := StrToInt(Copy(x, i - 2, 2));
  if x[i+1] = '-' then
  begin
    Sign[k] := -1;
    Latter[k] := StrToInt(Copy(x, i + 2, 2));
    Result := Copy(x, 1, i - 3) + FormatFloat('00', k + NumParam) + Copy(x, i + 4, 1000);
  end
  else
  begin
    Sign[k] := 1;
    Latter[k]   := StrToInt(Copy(x, i + 1, 2));
    Result := Copy(x, 1, i - 3) + FormatFloat('00', k + NumParam) + Copy(x, i + 3, 1000);
  end;
end;

function TMathsParser.CountString(Substr, S: String): Integer;
begin
  Result := 0;
  repeat
    if Pos(Substr, S) > 0 then
    begin
      S := Copy(S, Pos(Substr, S) + 1, 255);
      Result := Result + 1;
    end
    else
    Break;
  until Length(S) = 0;
end;

function TMathsParser.GetDoubleValue: Double;
var i: Integer;
begin
  P[1] := V1;
  P[2] := V2;
  P[3] := V3;
  P[4] := V4;
  P[5] := V5;
  P[6] := V6;
  P[7] := V7;
  P[8] := V8;
  for i := 1 to NumOperation do
  begin
    Case Operation[i] of
     1: P[NumParam+i] := Power(P[Former[i]], Sign[i] * P[Latter[i]]);
     2: P[NumParam+i] := Sign[i] * P[Former[i]] * P[Latter[i]];
     3: P[NumParam+i] := Sign[i] * P[Former[i]] / P[Latter[i]];
     4: P[NumParam+i] := P[Former[i]] + Sign[i] * P[Latter[i]];
     5: P[NumParam+i] := P[Former[i]] - Sign[i] * P[Latter[i]];
     6: P[NumParam+i] := Exp(P[Former[i]]);
     7: P[NumParam+i] := ln(P[Former[i]]);
     8: P[NumParam+i] := log10(P[Former[i]]);
     9: P[NumParam+i] := sin(P[Former[i]]);
    10: P[NumParam+i] := cos(P[Former[i]]);
    11: P[NumParam+i] := tan(P[Former[i]]);
    12: P[NumParam+i] := Abs(P[Former[i]]);
    13: P[NumParam+i] := ArcSin(P[Former[i]]);
    14: P[NumParam+i] := ArcCos(P[Former[i]]);
    15: P[NumParam+i] := ArcTan(P[Former[i]]);
    16: P[NumParam+i] := 1 - Erfc(P[Former[i]]);
    17: P[NumParam+i] := Erfc(P[Former[i]]);
    18: P[NumParam+i] := Sqrt(P[Former[i]]);
    end;
  end;
  Result := P[Former[NumOperation+1]];
end;

// From Numerical Recipes in Pascal, p185
function TMathsParser.Erfc(x: Double): Double;
var
   t, z, ans: Double;
begin
   z := abs(x);
   t := 1.0/(1.0+0.5*z);
   ans := t*exp(-z*z-1.26551223+t*(1.00002368+
      t*(0.37409196+t*(0.09678418+t*(-0.18628806+
      t*(0.27886807+t*(-1.13520398+t*(1.48851587+
      t*(-0.82215223+t*0.17087277)))))))));
   if (x >= 0.0) then
    Result := ans
   else
    Result := 2.0 - ans;
end;

procedure TMathsParser.ShowHelp;
begin
  ShowMessage(Msg[4]);
end;

procedure TMathsParser.DetectVariables(Expression: String; Exception: String = '');
var
  i, j, k, m: Integer;
  S: String;
  TempVName: array[1..50] of String;
  VName: array[1..8] of String;
  Excp: TStringList;
  IsNewVariable: Boolean;
begin
  if Expression = '' then Exit;
  Excp := TStringList.Create;
  Excp.CommaText := Exception;
  S := Expression;
  S := StringReplace(S, ' '  , '', [rfReplaceAll]);
  S := StringReplace(S, 'log10',   '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'sqrt' ,   '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'ln'   ,   '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'exp'  ,   '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'arcsin' , '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'arccos' , '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'arctan' , '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'sin'  ,   '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'cos'  ,   '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'tan'  ,   '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'abs'  ,   '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'erfc' ,   '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'erf'  ,   '', [rfReplaceAll, rfIgnoreCase]);
  S := StringReplace(S, 'pi'   ,   '', [rfReplaceAll, rfIgnoreCase]);
  for j := 1 to 8 do VName[j] := '';
  i := 1;
  j := 1;
  if S <> '' then
    repeat
      if ((Ord(S[i]) >= 65) and (Ord(S[i]) <= 90)) or
         ((Ord(S[i]) >= 97) and (Ord(S[i]) <= 122)) then
      begin
        TempVName[j] := GetValueName(Copy(S, i, 1000));
        for k := 0 to Excp.Count - 1 do
          if AnsiLowerCase(TempVName[j]) = AnsiLowerCase(Excp.Strings[k]) then
            TempVName[j] := '';
        if TempVName[j] <> '' then
        begin
         // S := StringReplace(S, TempVName[j], '', [rfReplaceAll, rfIgnoreCase]);
          i := i + Length(TempVName[j]);
          j := j + 1;
        end
        else
          i := i + 1;
      end
      else
        i := i + 1;
    until i >= Length(S) + 1;
    VName[1] := TempVName[1];
    m := 1;
    for i := 2 to j do
    begin
      IsNewVariable := True;
      for k := 1 to m do
        if TempVName[i] = VName[k] then
        begin
          IsNewVariable := False;
          Break;
        end;
      if IsNewVariable then
      begin
        m := m + 1;
        VName[m] := TempVName[i]
      end;
    end;
  V1Name := VName[1];
  V2Name := VName[2];
  V3Name := VName[3];
  V4Name := VName[4];
  V5Name := VName[5];
  V6Name := VName[6];
  V7Name := VName[7];
  V8Name := VName[8];
  Excp.Free;
end;

function TMathsParser.GetValueName(S: String): String;
var i: Integer;
begin
  S := S + ')';
  i := 0;
  repeat
    i := i + 1;
  until (Ord(S[i]) = 41) or (Ord(S[i]) = 42) or (Ord(S[i]) = 43)
     or (Ord(S[i]) = 45) or (Ord(S[i]) = 47) or (Ord(S[i]) = 125)
     or (Ord(S[i]) = 93) or (Ord(S[i]) = 94);
  Result := Copy(S, 1, i - 1);
end;

//-----------------------------------------------------------------------------------------
function TMathsParser.FindRoot(Expression: String; LowerBoundary, UpperBoundary, Accuracy: Double): Double;
var
  a, b, c, fa , fc: Double;
  i: Integer;
begin
  c := 0;
  SetExpression(Expression);
  a := LowerBoundary;
  b := UpperBoundary;
  fa := f(a);
  if fa * f(b) > 0 then
  begin
    ErrMsg := Msg[4];
    raise EInvalidInterval.Create(ErrMsg);
    Exit;
  end;
  if Accuracy > 0 then
    NumCalc := Ceil(ln(Abs(b - a) / Accuracy) / ln(2))
  else
    NumCalc := 20;
  for i := 1 to NumCalc do
  begin
    c := (a + b) / 2;
    fc := f(c);
    if fa * fc < 0 then
      b := c
    else
    begin
      a := c;
      fa := fc;
    end;
  end;
  Result := c;
end;

//===== Integration of a function using Simpson's method =====//
function TMathsParser.Integrate(Expression: String; LowerBoundary, UpperBoundary: Double; Division: Integer): Double;
var
  i, j: Integer;
 // dx, sigma1, sigma2: Double;
  d, dum: Double;
  x, y: array[0..1000] of Double;
begin
  SetExpression(Expression);
  Division := Ceil(2 * Trunc(Division / 2));
  for i := 0 to 4 do
  begin
    x[i] := LowerBoundary + i * (UpperBoundary - LowerBoundary) / 4;
    y[i] := f(x[i]);
  end;
  i := 5;
  repeat
    d := 0;
    j := 1;
    repeat
      dum := Abs(y[j] - 0.5 * (y[j-1] + y[j+1]));
      if dum > d then
      begin
        d := dum;
        k := j;
      end;
      j := j + 2;
    until (j >= i - 1);
    for j := i + 1 downto k + 1 do
    begin
      x[j] := x[j-2];
      y[j] := y[j-2];
    end;
    x[k+1] := x[k];
    y[k+1] := y[k];
    x[k]   := 0.5 * (x[k-1] + x[k+1]);
    x[k+2] := 0.5 * (x[k+3] + x[k+1]);
    y[k]   := f(x[k]);
    y[k+2] := f(x[k+2]);
    i := i + 2;
  until (i >= Division);
  Result := 0;
  i := 0;
  d := 1 / 3;
  repeat
    Result := Result + (x[i+1] - x[i]) * d * (y[i] + 4 * y[i+1] + y[i+2]);
    i := i + 2;
  until i >= Division - 1;
  {
  dx := (UpperBoundary - LowerBoundary) / Division;
  sigma1 := 0;
  for i := 2 to (Division div 2) do
    sigma1 := sigma1 + f(LowerBoundary + (2 * i - 2) * dx);
  sigma2 := 0;
  for i := 1 to (Division div 2) do
    sigma2 := sigma2 + f(LowerBoundary + (2 * i - 1) * dx);
  Result := dx / 3 * (f(LowerBoundary) + 2 * sigma1 + 4 * sigma2 + f(UpperBoundary));
  }
end;

//===== Nonlinear optimization for one parameter using Golden Section Method =====//
function TMathsParser.FindOptimumValue(Expression: String; LowerBoundary, UpperBoundary, Accuracy: Double; sign: Integer): Double;
var
  a0, a1, a2, a3, f1, f2: Double;
begin
  SetExpression(Expression);
  a0 := LowerBoundary;
  a3 := UpperBoundary;
  a1 := a0 + (a3 - a0) / 2.618;
  a2 := a0 + (a3 - a0) / 1.618;
  f1 := sign * f(a1);
  f2 := sign * f(a2);
  if sign = 0 then
    sign := -1;
  if Accuracy = 0 then
    Accuracy := Abs(a3 - a1) / 100000;
  NumCalc := 2;
  repeat
    if f1 > f2 then
    begin
      a3 := a2;
      a2 := a1;
      a1 := a0 + (a3 - a0) / 2.618;
      f2 := f1;
      f1 := sign * f(a1);
    end
    else
    begin
      a0 := a1;
      a1 := a2;
      a2 := a0 + (a3 - a0) / 1.618;
      f1 := f2;
      f2 := sign * f(a2);
    end;
    NumCalc := NumCalc + 1;
  until Abs(a3 - a0) < Accuracy;
  Result := (a1 + a2) / 2;
end;

function TMathsParser.f(x: Double): Double;
begin
  V1 := x;
  Result := Value;
end;

end.
