unit SlideList;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QButtons, QStdCtrls, QExtCtrls
  , IniFiles, Controls, StdCtrls, Buttons, ExtCtrls;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, ExtCtrls, IniFiles;
{$endif}

type
  TSlideProgram = class(TForm)
    Panel1: TPanel;
    btnOK: TButton;
    btnAdd: TButton;
    btnRemove: TButton;
    btnCancel: TButton;
    btnGoUp: TBitBtn;
    btnGoDown: TBitBtn;
    ListBox: TListBox;
    btnHelp: TButton;
    procedure btnAddClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ListBoxDblClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure btnGoUpClick(Sender: TObject);
    procedure btnGoDownClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
  private
  public
  end;

var
  SlideProgram: TSlideProgram;
  Msg: array[1..10] of String;

implementation

uses Grapher_Main, MyMsgDlg;

{$R *.xfm}

procedure TSlideProgram.FormCreate(Sender: TObject);
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
  i: Integer;
begin
  Msg[1] := 'Are you sure you want to remove this slide?';
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  Font.Assign(Main.Font);
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        Caption                  := ReadString(S, 'Caption','');
        btnAdd.Caption           := ReadString(S, 'btnAdd.Caption','');
        btnRemove.Caption        := ReadString(S, 'btnRemove.Caption','');
        btnCancel.Caption        := ReadString(S, 'btnCancel.Caption','');
        btnGoUp.Caption          := ReadString(S, 'btnGoUp.Caption','');
        btnGoDown.Caption        := ReadString(S, 'btnGoDown.Caption','');
        btnHelp.Caption          := ReadString(S, 'btnHelp.Caption','');
        Font.Name                := ReadString(S, 'Font.Name','');
        for i := 1 to 1 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>',   #13#10 , [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
end;

procedure TSlideProgram.FormShow(Sender: TObject);
var i : ShortInt;
begin
  ListBox.Items.Clear;
  for i := 1 to NumSlide do
  begin
    ListBox.Items.Add(SlideName[i]);
    {$ifdef MSWINDOWS}
    if ListBox.Canvas.TextWidth(SlideName[i]) > ListBox.Width then
      //ShowScrollBar(ListBox.Handle, SB_HORZ, True);
      ListBox.Perform(LB_SETHORIZONTALEXTENT, ListBox.Width * 2, 0);
    {$endif}
  end;
end;

procedure TSlideProgram.btnAddClick(Sender: TObject);
begin
  MouseUpEnabled := 0;
  with Main.OpenDialog1 do
  begin
    InitialDir := ExtractFileDir(SlideShowName);
    FilterIndex := 1;
    FileName := '';
    if Execute = True then
      ListBox.Items.Add(Main.GetRelativeDirectory(ExtractFileDir(SlideShowName), FileName));
  end;
end;

procedure TSlideProgram.btnOKClick(Sender: TObject);
var i : ShortInt;
begin
  with ListBox do
    for i := 0 to Items.Count - 1 do
      SlideName[i+1] := Items[i];
  NumSlide := ListBox.Items.Count;
  Modified[NumFigure] := 1;
  Close;
end;

procedure TSlideProgram.ListBoxDblClick(Sender: TObject);
begin
  if Main.puSlideShow.Checked = True then
  begin
    Main.OpenFigure(Main.GetAbsoluteDirectory(ExtractFileDir(SlideShowName), ListBox.Items[ListBox.ItemIndex]));
    SlideNumber := ListBox.ItemIndex + 1;
    Close;
  end;
end;

procedure TSlideProgram.btnRemoveClick(Sender: TObject);
var i : ShortInt;
begin
  {$ifdef Linux}
  if Application.MessageBox(PChar(Msg[2]), PChar('XYGrapher'), [smbYes, smbNo]) = smbNo then
  {$else}
  if Application.MessageBox(PChar(Msg[2]), PChar('XYGrapher'), MB_YESNO + MB_ICONQUESTION) = IDNO then
  {$endif}
    Exit;
  for i := ListBox.ItemIndex to ListBox.Items.Count - 2 do
    ListBox.Items[i] := ListBox.Items[i+1];
  ListBox.Items.Delete(ListBox.Items.Count - 1);
end;

procedure TSlideProgram.btnGoUpClick(Sender: TObject);
begin
  if ListBox.ItemIndex > 0 then
  begin
    dumstr := ListBox.Items[ListBox.ItemIndex];
    ListBox.Items[ListBox.ItemIndex]  := ListBox.Items[ListBox.ItemIndex-1];
    ListBox.Items[ListBox.ItemIndex-1]:= dumstr;
    ListBox.ItemIndex := ListBox.ItemIndex - 1;
  end;
end;

procedure TSlideProgram.btnGoDownClick(Sender: TObject);
begin
  if ListBox.ItemIndex < ListBox.Items.Count - 1 then
  begin
    dumstr := ListBox.Items[ListBox.ItemIndex];
    ListBox.Items[ListBox.ItemIndex]  := ListBox.Items[ListBox.ItemIndex+1];
    ListBox.Items[ListBox.ItemIndex+1]:= dumstr;
    ListBox.ItemIndex := ListBox.ItemIndex + 1;
  end;
end;

procedure TSlideProgram.btnHelpClick(Sender: TObject);
begin
  {$ifdef Linux}
  //Main.IndexClick(Sender);
  InvokeHelp;
  {$else}
  Keybd_Event(VK_F1, 0, 0, 0);
  {$endif}
end;

end.
